@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,34 +14,28 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
+from cStringIO import StringIO
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
     errors,
+    transport as _mod_transport,
     urlutils,
     )
 from bzrlib.bundle import serializer as _serializer
-from bzrlib.transport import (
-    do_catching_redirections,
-    get_transport,
-    )
+from bzrlib.merge_directive import MergeDirective
 """)
 from bzrlib.trace import note
 
 
-@deprecated_function(deprecated_in((1, 12, 0)))
-def read_bundle_from_url(url):
-    return read_mergeable_from_url(url, _do_directive=False)
-
-
 def read_mergeable_from_url(url, _do_directive=True, possible_transports=None):
     """Read mergable object from a given URL.
 
     :return: An object supporting get_target_revision.  Raises NotABundle if
         the target is not a mergeable type.
     """
-    child_transport = get_transport(url,
+    child_transport = _mod_transport.get_transport(url,
         possible_transports=possible_transports)
     transport = child_transport.clone('..')
     filename = transport.relpath(child_transport.base)
@@ -51,33 +45,22 @@ def read_mergeable_from_url(url, _do_directive=True, possible_transports=None):
 
 
 def read_mergeable_from_transport(transport, filename, _do_directive=True):
-    # All of this must be in the try/except
-    # Some transports cannot detect that we are trying to read a
-    # directory until we actually issue read() on the handle.
+    def get_bundle(transport):
+        return StringIO(transport.get_bytes(filename)), transport
+
+    def redirected_transport(transport, exception, redirection_notice):
+        note(redirection_notice)
+        url, filename = urlutils.split(exception.target,
+                                       exclude_trailing_slash=False)
+        if not filename:
+            raise errors.NotABundle('A directory cannot be a bundle')
+        return _mod_transport.get_transport(url)
+
     try:
-        def get_bundle(transport):
-            return transport.get(filename), transport
-
-        def redirected_transport(transport, exception, redirection_notice):
-            note(redirection_notice)
-            url, filename = urlutils.split(exception.target,
-                                           exclude_trailing_slash=False)
-            if not filename:
-                raise errors.NotABundle('A directory cannot be a bundle')
-            return get_transport(url)
-
-        try:
-            f, transport = do_catching_redirections(get_bundle, transport,
-                                                    redirected_transport)
-        except errors.TooManyRedirections:
-            raise errors.NotABundle(transport.clone(filename).base)
-
-        if _do_directive:
-            from bzrlib.merge_directive import MergeDirective
-            directive = MergeDirective.from_lines(f.readlines())
-            return directive, transport
-        else:
-            return _serializer.read_bundle(f), transport
+        bytef, transport = _mod_transport.do_catching_redirections(
+            get_bundle, transport, redirected_transport)
+    except errors.TooManyRedirections:
+        raise errors.NotABundle(transport.clone(filename).base)
     except (errors.ConnectionReset, errors.ConnectionError), e:
         raise
     except (errors.TransportError, errors.PathError), e:
@@ -91,6 +74,11 @@ def read_mergeable_from_transport(transport, filename, _do_directive=True):
         # StubSFTPServer does fail during get() (because of prefetch)
         # so it has an opportunity to translate the error.
         raise errors.NotABundle(str(e))
-    except errors.NotAMergeDirective:
-        f.seek(0)
-        return _serializer.read_bundle(f), transport
+
+    if _do_directive:
+        try:
+            return MergeDirective.from_lines(bytef), transport
+        except errors.NotAMergeDirective:
+            bytef.seek(0)
+
+    return _serializer.read_bundle(bytef), transport
