@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 import errno
 import os
-import time
 
 from bzrlib import errors, osutils
 from bzrlib.export import _export_iter_entries
@@ -26,27 +25,28 @@ from bzrlib.filters import (
     ContentFilterContext,
     filtered_output_bytes,
     )
-from bzrlib.trace import mutter
 
 
-def dir_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
-    """Export this tree to a new directory.
+def dir_exporter_generator(tree, dest, root, subdir=None, filtered=False,
+                           force_mtime=None, fileobj=None):
+    """Return a generator that exports this tree to a new directory.
 
     `dest` should either not exist or should be empty. If it does not exist it
     will be created holding the contents of this tree.
 
+    :param fileobj: Is not used in this exporter
+
     :note: If the export fails, the destination directory will be
            left in an incompletely exported state: export is not transactional.
     """
-    mutter('export version %r', tree)
     try:
         os.mkdir(dest)
     except OSError, e:
         if e.errno == errno.EEXIST:
             # check if directory empty
             if os.listdir(dest) != []:
-                raise errors.BzrError("Can't export tree to non-empty directory.")
+                raise errors.BzrError(
+                    "Can't export tree to non-empty directory.")
         else:
             raise
     # Iterate everything, building up the files we will want to export, and
@@ -66,17 +66,18 @@ def dir_exporter(tree, dest, root, subdir, filtered=False,
             try:
                 symlink_target = tree.get_symlink_target(ie.file_id)
                 os.symlink(symlink_target, fullpath)
-            except OSError,e:
+            except OSError, e:
                 raise errors.BzrError(
                     "Failed to create symlink %r -> %r, error: %s"
                     % (fullpath, symlink_target, e))
         else:
             raise errors.BzrError("don't know how to export {%s} of kind %r" %
                (ie.file_id, ie.kind))
+
+        yield
     # The data returned here can be in any order, but we've already created all
     # the directories
     flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | getattr(os, 'O_BINARY', 0)
-    now = time.time()
     for (relpath, executable), chunks in tree.iter_files_bytes(to_fetch):
         if filtered:
             filters = tree._content_filter_stack(relpath)
@@ -92,8 +93,14 @@ def dir_exporter(tree, dest, root, subdir, filtered=False,
             out.writelines(chunks)
         finally:
             out.close()
-        if per_file_timestamps:
-            mtime = tree.get_file_mtime(tree.path2id(relpath), relpath)
+        if force_mtime is not None:
+            mtime = force_mtime
         else:
-            mtime = now
+            if subdir is None:
+                file_id = tree.path2id(relpath)
+            else:
+                file_id = tree.path2id(osutils.pathjoin(subdir, relpath))
+            mtime = tree.get_file_mtime(file_id, relpath)
         os.utime(fullpath, (mtime, mtime))
+
+        yield
