@@ -14,11 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Export a Tree to a non-versioned directory.
+"""Export a Tree to a zip file.
 """
 
 import os
 import stat
+import sys
 import time
 import zipfile
 
@@ -33,29 +34,30 @@ from bzrlib.filters import (
 from bzrlib.trace import mutter
 
 
-# Windows expects this bit to be set in the 'external_attr' section
-# Or it won't consider the entry a directory
+# Windows expects this bit to be set in the 'external_attr' section,
+# or it won't consider the entry a directory.
 ZIP_DIRECTORY_BIT = (1 << 4)
 FILE_PERMISSIONS = (0644 << 16)
+DIR_PERMISSIONS = (0755 << 16)
 
 _FILE_ATTR = stat.S_IFREG | FILE_PERMISSIONS
-_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT
+_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT | DIR_PERMISSIONS
 
 
-def zip_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
+def zip_exporter_generator(tree, dest, root, subdir=None, filtered=False,
+    force_mtime=None, fileobj=None):
     """ Export this tree to a new zip file.
 
     `dest` will be created holding the contents of this tree; if it
     already exists, it will be overwritten".
     """
-    mutter('export version %r', tree)
-
-    now = time.localtime()[:6]
 
     compression = zipfile.ZIP_DEFLATED
+    if fileobj is not None:
+        dest = fileobj
+    elif dest == "-":
+        dest = sys.stdout
     zipf = zipfile.ZipFile(dest, "w", compression)
-
     try:
         for dp, ie in _export_iter_entries(tree, subdir):
             file_id = ie.file_id
@@ -63,15 +65,16 @@ def zip_exporter(tree, dest, root, subdir, filtered=False,
 
             # zipfile.ZipFile switches all paths to forward
             # slashes anyway, so just stick with that.
-            if per_file_timestamps:
-                mtime = tree.get_file_mtime(ie.file_id, dp)
+            if force_mtime is not None:
+                mtime = force_mtime
             else:
-                mtime = now
+                mtime = tree.get_file_mtime(ie.file_id, dp)
+            date_time = time.localtime(mtime)[:6]
             filename = osutils.pathjoin(root, dp).encode('utf8')
             if ie.kind == "file":
                 zinfo = zipfile.ZipInfo(
                             filename=filename,
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
                 if filtered:
@@ -89,17 +92,18 @@ def zip_exporter(tree, dest, root, subdir, filtered=False,
                 # not just empty files.
                 zinfo = zipfile.ZipInfo(
                             filename=filename + '/',
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _DIR_ATTR
-                zipf.writestr(zinfo,'')
+                zipf.writestr(zinfo, '')
             elif ie.kind == "symlink":
                 zinfo = zipfile.ZipInfo(
                             filename=(filename + '.lnk'),
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                zipf.writestr(zinfo, ie.symlink_target)
+                zipf.writestr(zinfo, tree.get_symlink_target(file_id))
+            yield
 
         zipf.close()
 
