@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 # TODO: Perhaps rather than mapping options and arguments back and
 # forth, we should just pass in the whole argv, and allow
@@ -20,10 +20,8 @@
 
 
 import os
-import sys
 
 from bzrlib.commands import Command
-from bzrlib.osutils import pathjoin
 
 
 class ExternalCommand(Command):
