@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,8 @@
 This includes waiting to import a module until it is actually used.
 
 Most commonly, the 'lazy_import' function is used to import other modules
-in an on-demand fashion. Typically use looks like:
+in an on-demand fashion. Typically use looks like::
+
     from bzrlib.lazy_import import lazy_import
     lazy_import(globals(), '''
     from bzrlib import (
@@ -30,14 +31,14 @@ in an on-demand fashion. Typically use looks like:
     import bzrlib.branch
     ''')
 
-    Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
-    objects which will be replaced with a real object on first use.
+Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
+objects which will be replaced with a real object on first use.
 
-    In general, it is best to only load modules in this way. This is because
-    it isn't safe to pass these variables to other functions before they
-    have been replaced. This is especially true for constants, sometimes
-    true for classes or functions (when used as a factory, or you want
-    to inherit from them).
+In general, it is best to only load modules in this way. This is because
+it isn't safe to pass these variables to other functions before they
+have been replaced. This is especially true for constants, sometimes
+true for classes or functions (when used as a factory, or you want
+to inherit from them).
 """
 
 
@@ -87,6 +88,9 @@ class ScopeReplacer(object):
                           " to another variable?",
                 extra=e)
         obj = factory(self, scope, name)
+        if obj is self:
+            raise errors.IllegalUseOfScopeReplacer(name, msg="Object tried"
+                " to replace itself, check it's not using its own scope.")
         if ScopeReplacer._should_proxy:
             object.__setattr__(self, '_real_obj', obj)
         scope[name] = obj
@@ -157,10 +161,14 @@ class ImportReplacer(ScopeReplacer):
             None, indicating the module is being imported.
         :param children: Children entries to be imported later.
             This should be a map of children specifications.
-            {'foo':(['bzrlib', 'foo'], None,
-                {'bar':(['bzrlib', 'foo', 'bar'], None {})})
-            }
-        Examples:
+            ::
+            
+                {'foo':(['bzrlib', 'foo'], None,
+                    {'bar':(['bzrlib', 'foo', 'bar'], None {})})
+                }
+
+        Examples::
+
             import foo => name='foo' module_path='foo',
                           member=None, children={}
             import foo.bar => name='foo' module_path='foo', member=None,
@@ -367,17 +375,18 @@ class ImportProcessor(object):
 def lazy_import(scope, text, lazy_import_class=None):
     """Create lazy imports for all of the imports in text.
 
-    This is typically used as something like:
-    from bzrlib.lazy_import import lazy_import
-    lazy_import(globals(), '''
-    from bzrlib import (
-        foo,
-        bar,
-        baz,
-        )
-    import bzrlib.branch
-    import bzrlib.transport
-    ''')
+    This is typically used as something like::
+
+        from bzrlib.lazy_import import lazy_import
+        lazy_import(globals(), '''
+        from bzrlib import (
+            foo,
+            bar,
+            baz,
+            )
+        import bzrlib.branch
+        import bzrlib.transport
+        ''')
 
     Then 'foo, bar, baz' and 'bzrlib' will exist as lazy-loaded
     objects which will be replaced with a real object on first use.
