@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
@@ -29,10 +29,10 @@ from bzrlib import (
 """)
 
 from bzrlib import (
-    log,
-    registry,
+    registry as _mod_registry,
     )
 
+
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
 
@@ -40,25 +40,25 @@ def _parse_revision_str(revstr):
     each revision specifier supplied.
 
     >>> _parse_revision_str('234')
-    [<RevisionSpec_revno 234>]
+    [<RevisionSpec_dwim 234>]
     >>> _parse_revision_str('234..567')
-    [<RevisionSpec_revno 234>, <RevisionSpec_revno 567>]
+    [<RevisionSpec_dwim 234>, <RevisionSpec_dwim 567>]
     >>> _parse_revision_str('..')
     [<RevisionSpec None>, <RevisionSpec None>]
     >>> _parse_revision_str('..234')
-    [<RevisionSpec None>, <RevisionSpec_revno 234>]
+    [<RevisionSpec None>, <RevisionSpec_dwim 234>]
     >>> _parse_revision_str('234..')
-    [<RevisionSpec_revno 234>, <RevisionSpec None>]
+    [<RevisionSpec_dwim 234>, <RevisionSpec None>]
     >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [<RevisionSpec_revno 234>, <RevisionSpec_revno 456>, <RevisionSpec_revno 789>]
+    [<RevisionSpec_dwim 234>, <RevisionSpec_dwim 456>, <RevisionSpec_dwim 789>]
     >>> _parse_revision_str('234....789') #Error ?
-    [<RevisionSpec_revno 234>, <RevisionSpec None>, <RevisionSpec_revno 789>]
+    [<RevisionSpec_dwim 234>, <RevisionSpec None>, <RevisionSpec_dwim 789>]
     >>> _parse_revision_str('revid:test@other.com-234234')
     [<RevisionSpec_revid revid:test@other.com-234234>]
     >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
     [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
     >>> _parse_revision_str('revid:test@other.com-234234..23')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revno 23>]
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_dwim 23>]
     >>> _parse_revision_str('date:2005-04-12')
     [<RevisionSpec_date date:2005-04-12>]
     >>> _parse_revision_str('date:2005-04-12 12:24:33')
@@ -68,27 +68,23 @@ def _parse_revision_str(revstr):
     >>> _parse_revision_str('date:2005-04-12,12:24:33')
     [<RevisionSpec_date date:2005-04-12,12:24:33>]
     >>> _parse_revision_str('-5..23')
-    [<RevisionSpec_revno -5>, <RevisionSpec_revno 23>]
+    [<RevisionSpec_dwim -5>, <RevisionSpec_dwim 23>]
     >>> _parse_revision_str('-5')
-    [<RevisionSpec_revno -5>]
+    [<RevisionSpec_dwim -5>]
     >>> _parse_revision_str('123a')
-    Traceback (most recent call last):
-      ...
-    NoSuchRevisionSpec: No namespace registered for string: '123a'
+    [<RevisionSpec_dwim 123a>]
     >>> _parse_revision_str('abc')
-    Traceback (most recent call last):
-      ...
-    NoSuchRevisionSpec: No namespace registered for string: 'abc'
+    [<RevisionSpec_dwim abc>]
     >>> _parse_revision_str('branch:../branch2')
     [<RevisionSpec_branch branch:../branch2>]
     >>> _parse_revision_str('branch:../../branch2')
     [<RevisionSpec_branch branch:../../branch2>]
     >>> _parse_revision_str('branch:../../branch2..23')
-    [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_revno 23>]
+    [<RevisionSpec_branch branch:../../branch2>, <RevisionSpec_dwim 23>]
     >>> _parse_revision_str('branch:..\\\\branch2')
     [<RevisionSpec_branch branch:..\\branch2>]
     >>> _parse_revision_str('branch:..\\\\..\\\\branch2..23')
-    [<RevisionSpec_branch branch:..\\..\\branch2>, <RevisionSpec_revno 23>]
+    [<RevisionSpec_branch branch:..\\..\\branch2>, <RevisionSpec_dwim 23>]
     """
     # TODO: Maybe move this into revisionspec.py
     revs = []
@@ -104,7 +100,7 @@ def _parse_change_str(revstr):
     parent of the revision.
 
     >>> _parse_change_str('123')
-    (<RevisionSpec_before before:123>, <RevisionSpec_revno 123>)
+    (<RevisionSpec_before before:123>, <RevisionSpec_dwim 123>)
     >>> _parse_change_str('123..124')
     Traceback (most recent call last):
       ...
@@ -136,7 +132,7 @@ def get_merge_type(typestring):
 
 class Option(object):
     """Description of a command line option
-    
+
     :ivar _short_name: If this option has a single-letter name, this is it.
     Otherwise None.
     """
@@ -150,16 +146,17 @@ class Option(object):
     OPTIONS = {}
 
     def __init__(self, name, help='', type=None, argname=None,
-                 short_name=None, param_name=None, custom_callback=None):
+                 short_name=None, param_name=None, custom_callback=None,
+                 hidden=False):
         """Make a new command option.
 
         :param name: regular name of the command, used in the double-dash
-            form and also as the parameter to the command's run() 
+            form and also as the parameter to the command's run()
             method (unless param_name is specified).
 
         :param help: help message displayed in command help
 
-        :param type: function called to parse the option argument, or 
+        :param type: function called to parse the option argument, or
             None (default) if this option doesn't take an argument.
 
         :param argname: name of option argument, if any
@@ -173,21 +170,25 @@ class Option(object):
         :param custom_callback: a callback routine to be called after normal
             processing. The signature of the callback routine is
             (option, name, new_value, parser).
+        :param hidden: If True, the option should be hidden in help and
+            documentation.
         """
         self.name = name
         self.help = help
         self.type = type
         self._short_name = short_name
         if type is None:
-            assert argname is None
+            if argname:
+                raise ValueError('argname not valid for booleans')
         elif argname is None:
             argname = 'ARG'
         self.argname = argname
         if param_name is None:
-            self._param_name = self.name
+            self._param_name = self.name.replace('-', '_')
         else:
             self._param_name = param_name
         self.custom_callback = custom_callback
+        self.hidden = hidden
 
     def short_name(self):
         if self._short_name:
@@ -207,24 +208,28 @@ class Option(object):
         option_strings = ['--%s' % self.name]
         if short_name is not None:
             option_strings.append('-%s' % short_name)
+        if self.hidden:
+            help = optparse.SUPPRESS_HELP
+        else:
+            help = self.help
         optargfn = self.type
         if optargfn is None:
-            parser.add_option(action='callback', 
-                              callback=self._optparse_bool_callback, 
+            parser.add_option(action='callback',
+                              callback=self._optparse_bool_callback,
                               callback_args=(True,),
-                              help=self.help,
+                              help=help,
                               *option_strings)
             negation_strings = ['--%s' % self.get_negation_name()]
-            parser.add_option(action='callback', 
-                              callback=self._optparse_bool_callback, 
+            parser.add_option(action='callback',
+                              callback=self._optparse_bool_callback,
                               callback_args=(False,),
                               help=optparse.SUPPRESS_HELP, *negation_strings)
         else:
-            parser.add_option(action='callback', 
-                              callback=self._optparse_callback, 
+            parser.add_option(action='callback',
+                              callback=self._optparse_callback,
                               type='string', metavar=self.argname.upper(),
-                              help=self.help,
-                              default=OptionParser.DEFAULT_VALUE, 
+                              help=help,
+                              default=OptionParser.DEFAULT_VALUE,
                               *option_strings)
 
     def _optparse_bool_callback(self, option, opt_str, value, parser, bool_v):
@@ -240,7 +245,7 @@ class Option(object):
 
     def iter_switches(self):
         """Iterate through the list of switches provided by the option
-        
+
         :return: an iterator of (name, short_name, argname, help)
         """
         argname =  self.argname
@@ -249,7 +254,7 @@ class Option(object):
         yield self.name, self.short_name(), argname, self.help
 
     def is_hidden(self, name):
-        return False
+        return self.hidden
 
 
 class ListOption(Option):
@@ -271,7 +276,7 @@ class ListOption(Option):
         parser.add_option(action='callback',
                           callback=self._optparse_callback,
                           type='string', metavar=self.argname.upper(),
-                          help=self.help, default=[],
+                          help=self.help, dest=self._param_name, default=[],
                           *option_strings)
 
     def _optparse_callback(self, option, opt, value, parser):
@@ -305,8 +310,9 @@ class RegistryOption(Option):
         else:
             return self.converter(value)
 
-    def __init__(self, name, help, registry, converter=None,
-        value_switches=False, title=None, enum_switch=True):
+    def __init__(self, name, help, registry=None, converter=None,
+        value_switches=False, title=None, enum_switch=True,
+        lazy_registry=None, short_name=None, short_value_switches=None):
         """
         Constructor.
 
@@ -320,17 +326,37 @@ class RegistryOption(Option):
             '--knit' can be used interchangeably.
         :param enum_switch: If true, a switch is provided with the option name,
             which takes a value.
+        :param lazy_registry: A tuple of (module name, attribute name) for a
+            registry to be lazily loaded.
+        :param short_name: The short name for the enum switch, if any
+        :param short_value_switches: A dict mapping values to short names
         """
-        Option.__init__(self, name, help, type=self.convert)
-        self.registry = registry
+        Option.__init__(self, name, help, type=self.convert, short_name=short_name)
+        self._registry = registry
+        if registry is None:
+            if lazy_registry is None:
+                raise AssertionError(
+                    'One of registry or lazy_registry must be given.')
+            self._lazy_registry = _mod_registry._LazyObjectGetter(
+                *lazy_registry)
+        if registry is not None and lazy_registry is not None:
+            raise AssertionError(
+                'registry and lazy_registry are mutually exclusive')
         self.name = name
         self.converter = converter
         self.value_switches = value_switches
         self.enum_switch = enum_switch
+        self.short_value_switches = short_value_switches
         self.title = title
         if self.title is None:
             self.title = name
 
+    @property
+    def registry(self):
+        if self._registry is None:
+            self._registry = self._lazy_registry.get_obj()
+        return self._registry
+
     @staticmethod
     def from_kwargs(name_, help=None, title=None, value_switches=False,
                     enum_switch=True, **kwargs):
@@ -338,12 +364,16 @@ class RegistryOption(Option):
 
         name, help, value_switches and enum_switch are passed to the
         RegistryOption constructor.  Any other keyword arguments are treated
-        as values for the option, and they value is treated as the help.
+        as values for the option, and their value is treated as the help.
         """
-        reg = registry.Registry()
-        for name, switch_help in kwargs.iteritems():
+        reg = _mod_registry.Registry()
+        for name, switch_help in sorted(kwargs.items()):
             name = name.replace('_', '-')
             reg.register(name, name, help=switch_help)
+            if not value_switches:
+                help = help + '  "' + name + '": ' + switch_help
+                if not help.endswith("."):
+                    help = help + "."
         return RegistryOption(name_, help, reg, title=title,
             value_switches=value_switches, enum_switch=enum_switch)
 
@@ -360,6 +390,10 @@ class RegistryOption(Option):
                     help = optparse.SUPPRESS_HELP
                 else:
                     help = self.registry.get_help(key)
+                if (self.short_value_switches and
+                    key in self.short_value_switches):
+                    option_strings.append('-%s' %
+                                          self.short_value_switches[key])
                 parser.add_option(action='callback',
                               callback=self._optparse_value_callback(key),
                                   help=help,
@@ -429,15 +463,10 @@ def _global_option(name, **kwargs):
     Option.OPTIONS[name] = Option(name, **kwargs)
 
 
-def _global_registry_option(name, help, registry, **kwargs):
+def _global_registry_option(name, help, registry=None, **kwargs):
     Option.OPTIONS[name] = RegistryOption(name, help, registry, **kwargs)
 
 
-class MergeTypeRegistry(registry.Registry):
-
-    pass
-
-
 # This is the verbosity level detected during command line parsing.
 # Note that the final value is dependent on the order in which the
 # various flags (verbose, quiet, no-verbose, no-quiet) are given.
@@ -466,6 +495,11 @@ def _verbosity_level_callback(option, opt_str, value, parser):
             _verbosity_level = -1
 
 
+class MergeTypeRegistry(_mod_registry.Registry):
+
+    pass
+
+
 _merge_type_registry = MergeTypeRegistry()
 _merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
                                    "Native diff3-style merge")
@@ -479,6 +513,8 @@ _merge_type_registry.register_lazy('lca', 'bzrlib.merge', 'LCAMerger',
 # Declare the standard options
 _standard_option('help', short_name='h',
                  help='Show help message.')
+_standard_option('usage',
+                 help='Show usage message and options.')
 _standard_option('verbose', short_name='v',
                  help='Display more information.',
                  custom_callback=_verbosity_level_callback)
@@ -501,6 +537,9 @@ _global_option('message', type=unicode,
                short_name='m',
                help='Message string.')
 _global_option('no-recurse')
+_global_option('null', short_name='0',
+                 help='Use an ASCII NUL (\\0) separator rather than '
+                      'a newline.')
 _global_option('profile',
                help='Show performance profiling information.')
 _global_option('revision',
@@ -514,16 +553,17 @@ _global_option('change',
                help='Select changes introduced by the specified revision. See also "help revisionspec".')
 _global_option('show-ids',
                help='Show internal object ids.')
-_global_option('timezone', 
+_global_option('timezone',
                type=str,
-               help='display timezone as local, original, or utc')
+               help='Display timezone as local, original, or utc.')
 _global_option('unbound')
 _global_option('version')
 _global_option('email')
 _global_option('update')
 _global_registry_option('log-format', "Use specified log format.",
-                        log.log_formatter_registry, value_switches=True,
-                        title='Log format')
+                        lazy_registry=('bzrlib.log', 'log_formatter_registry'),
+                        value_switches=True, title='Log format',
+                        short_value_switches={'short': 'S'})
 _global_option('long', help='Use detailed log format. Same as --log-format long',
                short_name='l')
 _global_option('short', help='Use moderately short log format. Same as --log-format short')
@@ -541,3 +581,9 @@ _global_option('kind', type=str)
 _global_option('dry-run',
                help="Show what would be done, but don't actually do anything.")
 _global_option('name-from-revision', help='The path name in the old tree.')
+_global_option('directory', short_name='d', type=unicode,
+               help='Branch to operate on, instead of working directory')
+
+diff_writer_registry = _mod_registry.Registry()
+diff_writer_registry.register('plain', lambda x: x, 'Plaintext diff output.')
+diff_writer_registry.default_key = 'plain'
