@@ -22,6 +22,9 @@ from bzrlib.bzrdir import (
     BzrDir,
     BzrDirFormat,
     BzrDirMetaFormat1,
+    BzrProber,
+    )
+from bzrlib.controldir import (
     network_format_registry,
     )
 from bzrlib.smart.request import (
@@ -44,10 +47,9 @@ class SmartServerRequestOpenBzrDir(SmartServerRequest):
             # clients that don't anticipate errors from this method.
             answer = 'no'
         else:
-            default_format = BzrDirFormat.get_default_format()
-            real_bzrdir = default_format.open(t, _found=True)
+            bzr_prober = BzrProber()
             try:
-                real_bzrdir._format.probe_transport(t)
+                bzr_prober.probe_transport(t)
             except (errors.NotBranchError, errors.UnknownFormatError):
                 answer = 'no'
             else:
@@ -84,7 +86,7 @@ class SmartServerRequestOpenBzrDir_2_1(SmartServerRequest):
 class SmartServerRequestBzrDir(SmartServerRequest):
 
     def do(self, path, *args):
-        """Open a BzrDir at path, and return self.do_bzrdir_request(*args)."""
+        """Open a BzrDir at path, and return `self.do_bzrdir_request(*args)`."""
         try:
             self._bzrdir = BzrDir.open_from_transport(
                 self.transport_from_client_path(path))
@@ -179,7 +181,8 @@ class SmartServerRequestCreateBranch(SmartServerRequestBzrDir):
 
         :param path: The path to the bzrdir.
         :param network_name: The network name of the branch type to create.
-        :return: (ok, network_name)
+        :return: ('ok', branch_format, repo_path, rich_root, tree_ref,
+            external_lookup, repo_format)
         """
         bzrdir = BzrDir.open_from_transport(
             self.transport_from_client_path(path))
