@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,19 +17,27 @@
 """Server for smart-server protocol."""
 
 import errno
+import os.path
 import socket
 import sys
 import threading
 
-from bzrlib.hooks import HookPoint, Hooks
+from bzrlib.hooks import Hooks
 from bzrlib import (
     errors,
     trace,
-    transport,
+    transport as _mod_transport,
 )
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib.smart import medium
+from bzrlib.transport import (
+    chroot,
+    pathfilter,
+    )
+from bzrlib import (
+    urlutils,
+    )
 """)
 
 
@@ -42,19 +50,25 @@ class SmartTCPServer(object):
     hooks: An instance of SmartServerHooks.
     """
 
-    def __init__(self, backing_transport, host='127.0.0.1', port=0,
-                 root_client_path='/'):
+    def __init__(self, backing_transport, root_client_path='/'):
         """Construct a new server.
 
         To actually start it running, call either start_background_thread or
         serve.
 
         :param backing_transport: The transport to serve.
-        :param host: Name of the interface to listen on.
-        :param port: TCP port to listen on, or 0 to allocate a transient port.
         :param root_client_path: The client path that will correspond to root
             of backing_transport.
         """
+        self.backing_transport = backing_transport
+        self.root_client_path = root_client_path
+
+    def start_server(self, host, port):
+        """Create the server listening socket.
+
+        :param host: Name of the interface to listen on.
+        :param port: TCP port to listen on, or 0 to allocate a transient port.
+        """
         # let connections timeout so that we get a chance to terminate
         # Keep a reference to the exceptions we want to catch because the socket
         # module's globals get set to None during interpreter shutdown.
@@ -80,21 +94,16 @@ class SmartTCPServer(object):
         self.port = self._sockname[1]
         self._server_socket.listen(1)
         self._server_socket.settimeout(1)
-        self.backing_transport = backing_transport
         self._started = threading.Event()
         self._stopped = threading.Event()
-        self.root_client_path = root_client_path
 
-    def serve(self, thread_name_suffix=''):
-        self._should_terminate = False
-        # for hooks we are letting code know that a server has started (and
-        # later stopped).
+    def _backing_urls(self):
         # There are three interesting urls:
         # The URL the server can be contacted on. (e.g. bzr://host/)
         # The URL that a commit done on the same machine as the server will
         # have within the servers space. (e.g. file:///home/user/source)
         # The URL that will be given to other hooks in the same process -
-        # the URL of the backing transport itself. (e.g. chroot+:///)
+        # the URL of the backing transport itself. (e.g. filtered-36195:///)
         # We need all three because:
         #  * other machines see the first
         #  * local commits on this machine should be able to be mapped to
@@ -104,15 +113,32 @@ class SmartTCPServer(object):
         # The latter two urls are different aliases to the servers url,
         # so we group those in a list - as there might be more aliases
         # in the future.
-        backing_urls = [self.backing_transport.base]
+        urls = [self.backing_transport.base]
         try:
-            backing_urls.append(self.backing_transport.external_url())
+            urls.append(self.backing_transport.external_url())
         except errors.InProcessTransport:
             pass
+        return urls
+
+    def run_server_started_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
         for hook in SmartTCPServer.hooks['server_started']:
             hook(backing_urls, self.get_url())
         for hook in SmartTCPServer.hooks['server_started_ex']:
             hook(backing_urls, self)
+
+    def run_server_stopped_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
+        for hook in SmartTCPServer.hooks['server_stopped']:
+            hook(backing_urls, self.get_url())
+
+    def serve(self, thread_name_suffix=''):
+        self._should_terminate = False
+        # for hooks we are letting code know that a server has started (and
+        # later stopped).
+        self.run_server_started_hooks()
         self._started.set()
         try:
             try:
@@ -129,13 +155,14 @@ class SmartTCPServer(object):
                         if e.args[0] != errno.EBADF:
                             trace.warning("listening socket error: %s", e)
                     else:
+                        if self._should_terminate:
+                            break
                         self.serve_conn(conn, thread_name_suffix)
             except KeyboardInterrupt:
                 # dont log when CTRL-C'd.
                 raise
             except Exception, e:
-                trace.error("Unhandled smart server error.")
-                trace.log_exception_quietly()
+                trace.report_exception(sys.exc_info(), sys.stderr)
                 raise
         finally:
             self._stopped.set()
@@ -145,12 +172,11 @@ class SmartTCPServer(object):
             except self._socket_error:
                 # ignore errors on close
                 pass
-            for hook in SmartTCPServer.hooks['server_stopped']:
-                hook(backing_urls, self.get_url())
+            self.run_server_stopped_hooks()
 
     def get_url(self):
         """Return the url of the server"""
-        return "bzr://%s:%d/" % self._sockname
+        return "bzr://%s:%s/" % (self._sockname[0], self._sockname[1])
 
     def serve_conn(self, conn, thread_name_suffix):
         # For WIN32, where the timeout value from the listening socket
@@ -162,8 +188,12 @@ class SmartTCPServer(object):
         thread_name = 'smart-server-child' + thread_name_suffix
         connection_thread = threading.Thread(
             None, handler.serve, name=thread_name)
+        # FIXME: This thread is never joined, it should at least be collected
+        # somewhere so that tests that want to check for leaked threads can get
+        # rid of them -- vila 20100531
         connection_thread.setDaemon(True)
         connection_thread.start()
+        return connection_thread
 
     def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
@@ -209,138 +239,161 @@ class SmartServerHooks(Hooks):
         These are all empty initially, because by default nothing should get
         notified.
         """
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('server_started',
+        Hooks.__init__(self, "bzrlib.smart.server", "SmartTCPServer.hooks")
+        self.add_hook('server_started',
             "Called by the bzr server when it starts serving a directory. "
             "server_started is called with (backing urls, public url), "
             "where backing_url is a list of URLs giving the "
             "server-specific directory locations, and public_url is the "
-            "public URL for the directory being served.", (0, 16), None))
-        self.create_hook(HookPoint('server_started_ex',
+            "public URL for the directory being served.", (0, 16))
+        self.add_hook('server_started_ex',
             "Called by the bzr server when it starts serving a directory. "
             "server_started is called with (backing_urls, server_obj).",
-            (1, 17), None))
-        self.create_hook(HookPoint('server_stopped',
+            (1, 17))
+        self.add_hook('server_stopped',
             "Called by the bzr server when it stops serving a directory. "
             "server_stopped is called with the same parameters as the "
-            "server_started hook: (backing_urls, public_url).", (0, 16), None))
+            "server_started hook: (backing_urls, public_url).", (0, 16))
+        self.add_hook('server_exception',
+            "Called by the bzr server when an exception occurs. "
+            "server_exception is called with the sys.exc_info() tuple "
+            "return true for the hook if the exception has been handled, "
+            "in which case the server will exit normally.", (2, 4))
 
 SmartTCPServer.hooks = SmartServerHooks()
 
 
-class SmartTCPServer_for_testing(SmartTCPServer):
-    """Server suitable for use by transport tests.
-
-    This server is backed by the process's cwd.
+def _local_path_for_transport(transport):
+    """Return a local path for transport, if reasonably possible.
+    
+    This function works even if transport's url has a "readonly+" prefix,
+    unlike local_path_from_url.
+    
+    This essentially recovers the --directory argument the user passed to "bzr
+    serve" from the transport passed to serve_bzr.
     """
-
-    def __init__(self, thread_name_suffix=''):
-        SmartTCPServer.__init__(self, None)
-        self.client_path_extra = None
-        self.thread_name_suffix = thread_name_suffix
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        return transport.get_transport(backing_transport_server.get_url())
-
-    def setUp(self, backing_transport_server=None,
-              client_path_extra='/extra/'):
-        """Set up server for testing.
-
-        :param backing_transport_server: backing server to use.  If not
-            specified, a LocalURLServer at the current working directory will
-            be used.
-        :param client_path_extra: a path segment starting with '/' to append to
-            the root URL for this server.  For instance, a value of '/foo/bar/'
-            will mean the root of the backing transport will be published at a
-            URL like `bzr://127.0.0.1:nnnn/foo/bar/`, rather than
-            `bzr://127.0.0.1:nnnn/`.  Default value is `extra`, so that tests
-            by default will fail unless they do the necessary path translation.
-        """
-        if not client_path_extra.startswith('/'):
-            raise ValueError(client_path_extra)
-        from bzrlib.transport.chroot import ChrootServer
-        if backing_transport_server is None:
-            from bzrlib.transport.local import LocalURLServer
-            backing_transport_server = LocalURLServer()
-        self.chroot_server = ChrootServer(
-            self.get_backing_transport(backing_transport_server))
-        self.chroot_server.setUp()
-        self.backing_transport = transport.get_transport(
-            self.chroot_server.get_url())
-        self.root_client_path = self.client_path_extra = client_path_extra
-        self.start_background_thread(self.thread_name_suffix)
-
-    def tearDown(self):
-        self.stop_background_thread()
-        self.chroot_server.tearDown()
-
-    def get_url(self):
-        url = super(SmartTCPServer_for_testing, self).get_url()
-        return url[:-1] + self.client_path_extra
-
-    def get_bogus_url(self):
-        """Return a URL which will fail to connect"""
-        return 'bzr://127.0.0.1:1/'
-
-
-class ReadonlySmartTCPServer_for_testing(SmartTCPServer_for_testing):
-    """Get a readonly server for testing."""
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        url = 'readonly+' + backing_transport_server.get_url()
-        return transport.get_transport(url)
-
-
-class SmartTCPServer_for_testing_v2_only(SmartTCPServer_for_testing):
-    """A variation of SmartTCPServer_for_testing that limits the client to
-    using RPCs in protocol v2 (i.e. bzr <= 1.5).
-    """
-
-    def get_url(self):
-        url = super(SmartTCPServer_for_testing_v2_only, self).get_url()
-        url = 'bzr-v2://' + url[len('bzr://'):]
-        return url
-
-
-class ReadonlySmartTCPServer_for_testing_v2_only(SmartTCPServer_for_testing_v2_only):
-    """Get a readonly server for testing."""
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        url = 'readonly+' + backing_transport_server.get_url()
-        return transport.get_transport(url)
-
-
-def serve_bzr(transport, host=None, port=None, inet=False):
-    from bzrlib import lockdir, ui
-    from bzrlib.transport import get_transport
-    from bzrlib.transport.chroot import ChrootServer
-    chroot_server = ChrootServer(transport)
-    chroot_server.setUp()
-    transport = get_transport(chroot_server.get_url())
-    if inet:
-        smart_server = medium.SmartServerPipeStreamMedium(
-            sys.stdin, sys.stdout, transport)
-    else:
-        if host is None:
-            host = medium.BZR_DEFAULT_INTERFACE
-        if port is None:
-            port = medium.BZR_DEFAULT_PORT
-        smart_server = SmartTCPServer(transport, host=host, port=port)
-        trace.note('listening on port: %s' % smart_server.port)
-    # For the duration of this server, no UI output is permitted. note
-    # that this may cause problems with blackbox tests. This should be
-    # changed with care though, as we dont want to use bandwidth sending
-    # progress over stderr to smart server clients!
-    old_factory = ui.ui_factory
-    old_lockdir_timeout = lockdir._DEFAULT_TIMEOUT_SECONDS
     try:
+        base_url = transport.external_url()
+    except (errors.InProcessTransport, NotImplementedError):
+        return None
+    else:
+        # Strip readonly prefix
+        if base_url.startswith('readonly+'):
+            base_url = base_url[len('readonly+'):]
+        try:
+            return urlutils.local_path_from_url(base_url)
+        except errors.InvalidURL:
+            return None
+
+
+class BzrServerFactory(object):
+    """Helper class for serve_bzr."""
+
+    def __init__(self, userdir_expander=None, get_base_path=None):
+        self.cleanups = []
+        self.base_path = None
+        self.backing_transport = None
+        if userdir_expander is None:
+            userdir_expander = os.path.expanduser
+        self.userdir_expander = userdir_expander
+        if get_base_path is None:
+            get_base_path = _local_path_for_transport
+        self.get_base_path = get_base_path
+
+    def _expand_userdirs(self, path):
+        """Translate /~/ or /~user/ to e.g. /home/foo, using
+        self.userdir_expander (os.path.expanduser by default).
+
+        If the translated path would fall outside base_path, or the path does
+        not start with ~, then no translation is applied.
+
+        If the path is inside, it is adjusted to be relative to the base path.
+
+        e.g. if base_path is /home, and the expanded path is /home/joe, then
+        the translated path is joe.
+        """
+        result = path
+        if path.startswith('~'):
+            expanded = self.userdir_expander(path)
+            if not expanded.endswith('/'):
+                expanded += '/'
+            if expanded.startswith(self.base_path):
+                result = expanded[len(self.base_path):]
+        return result
+
+    def _make_expand_userdirs_filter(self, transport):
+        return pathfilter.PathFilteringServer(transport, self._expand_userdirs)
+
+    def _make_backing_transport(self, transport):
+        """Chroot transport, and decorate with userdir expander."""
+        self.base_path = self.get_base_path(transport)
+        chroot_server = chroot.ChrootServer(transport)
+        chroot_server.start_server()
+        self.cleanups.append(chroot_server.stop_server)
+        transport = _mod_transport.get_transport(chroot_server.get_url())
+        if self.base_path is not None:
+            # Decorate the server's backing transport with a filter that can
+            # expand homedirs.
+            expand_userdirs = self._make_expand_userdirs_filter(transport)
+            expand_userdirs.start_server()
+            self.cleanups.append(expand_userdirs.stop_server)
+            transport = _mod_transport.get_transport(expand_userdirs.get_url())
+        self.transport = transport
+
+    def _make_smart_server(self, host, port, inet):
+        if inet:
+            smart_server = medium.SmartServerPipeStreamMedium(
+                sys.stdin, sys.stdout, self.transport)
+        else:
+            if host is None:
+                host = medium.BZR_DEFAULT_INTERFACE
+            if port is None:
+                port = medium.BZR_DEFAULT_PORT
+            smart_server = SmartTCPServer(self.transport)
+            smart_server.start_server(host, port)
+            trace.note('listening on port: %s' % smart_server.port)
+        self.smart_server = smart_server
+
+    def _change_globals(self):
+        from bzrlib import lockdir, ui
+        # For the duration of this server, no UI output is permitted. note
+        # that this may cause problems with blackbox tests. This should be
+        # changed with care though, as we dont want to use bandwidth sending
+        # progress over stderr to smart server clients!
+        old_factory = ui.ui_factory
+        old_lockdir_timeout = lockdir._DEFAULT_TIMEOUT_SECONDS
+        def restore_default_ui_factory_and_lockdir_timeout():
+            ui.ui_factory = old_factory
+            lockdir._DEFAULT_TIMEOUT_SECONDS = old_lockdir_timeout
+        self.cleanups.append(restore_default_ui_factory_and_lockdir_timeout)
         ui.ui_factory = ui.SilentUIFactory()
         lockdir._DEFAULT_TIMEOUT_SECONDS = 0
-        smart_server.serve()
-    finally:
-        ui.ui_factory = old_factory
-        lockdir._DEFAULT_TIMEOUT_SECONDS = old_lockdir_timeout
 
+    def set_up(self, transport, host, port, inet):
+        self._make_backing_transport(transport)
+        self._make_smart_server(host, port, inet)
+        self._change_globals()
+
+    def tear_down(self):
+        for cleanup in reversed(self.cleanups):
+            cleanup()
+
+def serve_bzr(transport, host=None, port=None, inet=False):
+    """This is the default implementation of 'bzr serve'.
+    
+    It creates a TCP or pipe smart server on 'transport, and runs it.  The
+    transport will be decorated with a chroot and pathfilter (using
+    os.path.expanduser).
+    """
+    bzr_server = BzrServerFactory()
+    try:
+        bzr_server.set_up(transport, host, port, inet)
+        bzr_server.smart_server.serve()
+    except:
+        hook_caught_exception = False
+        for hook in SmartTCPServer.hooks['server_exception']:
+            hook_caught_exception = hook(sys.exc_info())
+        if not hook_caught_exception:
+            raise
+    finally:
+        bzr_server.tear_down()
