@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,12 +25,21 @@ when the branch is opened.  Clients should typically do
 # NOTE: I was going to call this tags.py, but vim seems to think all files
 # called tags* are ctags files... mbp 20070220.
 
+from bzrlib.registry import Registry
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import itertools
+import re
+import sys
 
 from bzrlib import (
     bencode,
+    cleanup,
     errors,
+    symbol_versioning,
     trace,
     )
+""")
 
 
 class _Tags(object):
@@ -57,7 +66,7 @@ class DisabledTags(_Tags):
     lookup_tag = _not_supported
     delete_tag = _not_supported
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         # we never have anything to copy
         pass
 
@@ -177,7 +186,7 @@ class BasicTags(_Tags):
             raise ValueError("failed to deserialize tag dictionary %r: %s"
                 % (tag_content, e))
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         """Copy tags between repositories if necessary and possible.
 
         This method has common command-line behaviour about handling
@@ -188,11 +197,19 @@ class BasicTags(_Tags):
 
         :param to_tags: Branch to receive these tags
         :param overwrite: Overwrite conflicting tags in the target branch
+        :param ignore_master: Do not modify the tags in the target's master
+            branch (if any).  Default is false (so the master will be updated).
+            New in bzr 2.3.
 
-        :returns: A list of tags that conflicted, each of which is
+        :returns: A set of tags that conflicted, each of which is
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
+        operation = cleanup.OperationWithCleanups(self._merge_to_operation)
+        return operation.run(to_tags, overwrite, ignore_master)
+
+    def _merge_to_operation(self, operation, to_tags, overwrite, ignore_master):
+        add_cleanup = operation.add_cleanup
         if self.branch == to_tags.branch:
             return
         if not self.branch.supports_tags():
@@ -203,15 +220,39 @@ class BasicTags(_Tags):
             # no tags in the source, and we don't want to clobber anything
             # that's in the destination
             return
-        to_tags.branch.lock_write()
-        try:
-            dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(source_dict, dest_dict,
-                                                     overwrite)
-            if result != dest_dict:
-                to_tags._set_tag_dict(result)
-        finally:
-            to_tags.branch.unlock()
+        # We merge_to both master and child individually.
+        #
+        # It's possible for master and child to have differing sets of
+        # tags, in which case it's possible to have different sets of
+        # conflicts.  We report the union of both conflict sets.  In
+        # that case it's likely the child and master have accepted
+        # different tags from the source, which may be a surprising result, but
+        # the best we can do in the circumstances.
+        #
+        # Ideally we'd improve this API to report the different conflicts
+        # more clearly to the caller, but we don't want to break plugins
+        # such as bzr-builddeb that use this API.
+        add_cleanup(to_tags.branch.lock_write().unlock)
+        if ignore_master:
+            master = None
+        else:
+            master = to_tags.branch.get_master_branch()
+        if master is not None:
+            add_cleanup(master.lock_write().unlock)
+        conflicts = self._merge_to(to_tags, source_dict, overwrite)
+        if master is not None:
+            conflicts += self._merge_to(master.tags, source_dict,
+                overwrite)
+        # We use set() to remove any duplicate conflicts from the master
+        # branch.
+        return set(conflicts)
+
+    def _merge_to(self, to_tags, source_dict, overwrite):
+        dest_dict = to_tags.get_tag_dict()
+        result, conflicts = self._reconcile_tags(source_dict, dest_dict,
+                                                 overwrite)
+        if result != dest_dict:
+            to_tags._set_tag_dict(result)
         return conflicts
 
     def rename_revisions(self, rename_map):
@@ -228,11 +269,11 @@ class BasicTags(_Tags):
     def _reconcile_tags(self, source_dict, dest_dict, overwrite):
         """Do a two-way merge of two tag dictionaries.
 
-        only in source => source value
-        only in destination => destination value
-        same definitions => that
-        different definitions => if overwrite is False, keep destination
-            value and give a warning, otherwise use the source value
+        * only in source => source value
+        * only in destination => destination value
+        * same definitions => that
+        * different definitions => if overwrite is False, keep destination
+          value and give a warning, otherwise use the source value
 
         :returns: (result_dict,
             [(conflicting_tag, source_target, dest_target)])
@@ -249,6 +290,74 @@ class BasicTags(_Tags):
         return result, conflicts
 
 
-def _merge_tags_if_possible(from_branch, to_branch):
-    from_branch.tags.merge_to(to_branch.tags)
+def _merge_tags_if_possible(from_branch, to_branch, ignore_master=False):
+    # Try hard to support merge_to implementations that don't expect
+    # 'ignore_master' (new in bzr 2.3).  First, if the flag isn't set then we
+    # can safely avoid passing ignore_master at all.
+    if not ignore_master:
+        from_branch.tags.merge_to(to_branch.tags)
+        return
+    # If the flag is set, try to pass it, but be ready to catch TypeError.
+    try:
+        from_branch.tags.merge_to(to_branch.tags, ignore_master=ignore_master)
+    except TypeError:
+        # Probably this implementation of 'merge_to' is from a plugin that
+        # doesn't expect the 'ignore_master' keyword argument (e.g. bzr-svn
+        # 1.0.4).  There's a small risk that the TypeError is actually caused
+        # by a completely different problem (which is why we don't catch it for
+        # the ignore_master=False case), but even then there's probably no harm
+        # in calling a second time.
+        symbol_versioning.warn(
+            symbol_versioning.deprecated_in((2,3)) % (
+                "Tags.merge_to (of %r) that doesn't accept ignore_master kwarg"
+                % (from_branch.tags,),),
+            DeprecationWarning)
+        from_branch.tags.merge_to(to_branch.tags)
 
+
+def sort_natural(branch, tags):
+    """Sort tags, with numeric substrings as numbers.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    def natural_sort_key(tag):
+        return [f(s) for f,s in
+                zip(itertools.cycle((unicode.lower,int)),
+                                    re.split('([0-9]+)', tag[0]))]
+    tags.sort(key=natural_sort_key)
+
+
+def sort_alpha(branch, tags):
+    """Sort tags lexicographically, in place.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    tags.sort()
+
+
+def sort_time(branch, tags):
+    """Sort tags by time inline.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    timestamps = {}
+    for tag, revid in tags:
+        try:
+            revobj = branch.repository.get_revision(revid)
+        except errors.NoSuchRevision:
+            timestamp = sys.maxint # place them at the end
+        else:
+            timestamp = revobj.timestamp
+        timestamps[revid] = timestamp
+    tags.sort(key=lambda x: timestamps[x[1]])
+
+
+tag_sort_methods = Registry()
+tag_sort_methods.register("natural", sort_natural,
+    'Sort numeric substrings as numbers. (default)')
+tag_sort_methods.register("alpha", sort_alpha, 'Sort tags lexicographically.')
+tag_sort_methods.register("time", sort_time, 'Sort tags chronologically.')
+tag_sort_methods.default_key = "natural"
