@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,25 +25,22 @@ from bzrlib import (
     errors,
     info,
     osutils,
+    tests,
     upgrade,
     urlutils,
     )
-from bzrlib.osutils import format_date
-from bzrlib.tests import TestSkipped
-from bzrlib.tests.blackbox import ExternalBase
+from bzrlib.transport import memory
 
 
-class TestInfo(ExternalBase):
+class TestInfo(tests.TestCaseWithTransport):
 
     def setUp(self):
-        ExternalBase.setUp(self)
-        self._repo_strings = "2a or development-subtree"
+        super(TestInfo, self).setUp()
+        self._repo_strings = "2a"
 
     def test_info_non_existing(self):
-        if sys.platform == "win32":
-            location = "C:/i/do/not/exist/"
-        else:
-            location = "/i/do/not/exist/"
+        self.vfs_transport_factory = memory.MemoryServer
+        location = self.get_url()
         out, err = self.run_bzr('info '+location, retcode=3)
         self.assertEqual(out, '')
         self.assertEqual(err, 'bzr: ERROR: Not a branch: "%s".\n' % location)
@@ -52,14 +49,14 @@ class TestInfo(ExternalBase):
         transport = self.get_transport()
 
         # Create initial standalone branch
-        tree1 = self.make_branch_and_tree('standalone', 'weave')
+        tree1 = self.make_branch_and_tree('standalone', 'knit')
         self.build_tree(['standalone/a'])
         tree1.add('a')
         branch1 = tree1.branch
 
         out, err = self.run_bzr('info standalone')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 """, out)
@@ -68,15 +65,15 @@ Location:
         # Standalone branch - verbose mode
         out, err = self.run_bzr('info standalone -v')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          0 unchanged
@@ -99,15 +96,15 @@ Repository:
         # Standalone branch - really verbose mode
         out, err = self.run_bzr('info standalone -vv')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          0 unchanged
@@ -129,7 +126,7 @@ Repository:
         self.assertEqual('', err)
         tree1.commit('commit one')
         rev = branch1.repository.get_revision(branch1.revision_history()[0])
-        datestring_first = format_date(rev.timestamp, rev.timezone)
+        datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Branch standalone with push location
         branch2 = branch1.bzrdir.sprout('branch').open_branch()
@@ -137,7 +134,7 @@ Repository:
 
         out, err = self.run_bzr('info branch')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -149,7 +146,7 @@ Related branches:
 
         out, err = self.run_bzr('info branch --verbose')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -158,10 +155,10 @@ Related branches:
   parent branch: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -215,7 +212,7 @@ In the working tree:
          0 added
          0 removed
          0 renamed
-         1 unknown
+         0 unknown
          0 ignored
          0 versioned subdirectories
 
@@ -279,6 +276,10 @@ Repository:
         tree5 = branch1.create_checkout('lightcheckout', lightweight=True)
         branch5 = tree5.branch
         out, err = self.run_bzr('info -v lightcheckout')
+        if "metaweave" in bzrdir.format_registry:
+            format_description = "knit or metaweave"
+        else:
+            format_description = "knit"
         self.assertEqualDiff(
 """Lightweight checkout (format: %s)
 Location:
@@ -287,9 +288,9 @@ Location:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 6
-        branch: Branch format 4
-    repository: Weave repository format 6
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -309,7 +310,7 @@ Branch history:
 
 Repository:
          1 revision
-""" % (self._repo_strings, datestring_first, datestring_first,), out)
+""" % (format_description, datestring_first, datestring_first,), out)
         self.assertEqual('', err)
 
         # Update initial standalone branch
@@ -317,12 +318,12 @@ Repository:
         tree1.add('b')
         tree1.commit('commit two')
         rev = branch1.repository.get_revision(branch1.revision_history()[-1])
-        datestring_last = format_date(rev.timestamp, rev.timezone)
+        datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Out of date branched standalone branch will not be detected
         out, err = self.run_bzr('info -v branch')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -331,10 +332,10 @@ Related branches:
   parent branch: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -383,7 +384,7 @@ In the working tree:
          0 added
          0 removed
          0 renamed
-         1 unknown
+         0 unknown
          0 ignored
          0 versioned subdirectories
 
@@ -449,9 +450,9 @@ Location:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 6
-        branch: Branch format 4
-    repository: Weave repository format 6
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 Working tree is out of date: missing 1 revision.
 
@@ -473,7 +474,7 @@ Branch history:
 
 Repository:
          2 revisions
-""" % (self._repo_strings, datestring_first, datestring_last,), out)
+""" % (format_description, datestring_first, datestring_last,), out)
         self.assertEqual('', err)
 
     def test_info_standalone_no_tree(self):
@@ -571,7 +572,7 @@ Repository:
         tree2.add('a')
         tree2.commit('commit one')
         rev = repo.get_revision(branch2.revision_history()[0])
-        datestring_first = format_date(rev.timestamp, rev.timezone)
+        datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Lightweight checkout (format: %s)
@@ -690,7 +691,7 @@ Repository:
 
         # Out of date lightweight checkout
         rev = repo.get_revision(branch1.revision_history()[-1])
-        datestring_last = format_date(rev.timestamp, rev.timezone)
+        datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Lightweight checkout (format: %s)
@@ -846,7 +847,7 @@ Repository:
         tree1.add('a')
         tree1.commit('commit one')
         rev = repo.get_revision(branch1.revision_history()[0])
-        datestring_first = format_date(rev.timestamp, rev.timezone)
+        datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info -v repo/branch1')
         self.assertEqualDiff(
 """Repository tree (format: knit)
@@ -1231,7 +1232,8 @@ Repository:
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')
         lco_dir = bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
-        branch.BranchReferenceFormat().initialize(lco_dir, co_branch)
+        branch.BranchReferenceFormat().initialize(lco_dir,
+            target_branch=co_branch)
         lco_dir.create_workingtree()
         lco_tree = lco_dir.open_workingtree()
 
@@ -1325,85 +1327,6 @@ Repository:
             self.knownFailure('Win32 cannot run "bzr info"'
                               ' when the tree is locked.')
 
-    def test_info_locking_oslocks(self):
-        if sys.platform == "win32":
-            raise TestSkipped("don't use oslocks on win32 in unix manner")
-        self.thisFailsStrictLockCheck()
-
-        tree = self.make_branch_and_tree('branch',
-                                         format=bzrdir.BzrDirFormat6())
-
-        # Test all permutations of locking the working tree, branch and repository
-        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
-        # implemented by raising NotImplementedError and get_physical_lock_status()
-        # always returns false. This makes bzr info hide the lock status.  (Olaf)
-        # W B R
-
-        # U U U
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        # L L L
-        tree.lock_write()
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        tree.unlock()
-
     def test_info_stacked(self):
         # We have a mainline
         trunk_tree = self.make_branch_and_tree('mainline',
