@@ -19,7 +19,6 @@
 
 from itertools import izip
 import os
-import re
 
 from bzrlib import (
     branchbuilder,
@@ -29,8 +28,8 @@ from bzrlib import (
     tests,
     )
 from bzrlib.tests import (
-    script,
     test_log,
+    features,
     )
 
 
@@ -77,6 +76,8 @@ class TestLogWithLogCatcher(TestLog):
                 self.log_catcher = test_log.LogCatcher(*args, **kwargs)
                 # Always return our own log formatter
                 return self.log_catcher
+        # Break cycle with closure over self on cleanup by removing method
+        self.addCleanup(setattr, MyLogFormatter, "__new__", None)
 
         def getme(branch):
                 # Always return our own log formatter class hijacking the
@@ -442,6 +443,25 @@ class TestLogTags(TestLog):
         self.assertContainsRe(log, r'tags: tag1')
 
 
+class TestLogSignatures(TestLog):
+
+    def test_log_with_signatures(self):
+        self.requireFeature(features.gpgme)
+
+        tree = self.make_linear_branch(format='dirstate-tags')
+
+        log = self.run_bzr("log --signatures")[0]
+        self.assertTrue('signature: no signature' in log)
+
+    def test_log_without_signatures(self):
+        self.requireFeature(features.gpgme)
+
+        tree = self.make_linear_branch(format='dirstate-tags')
+
+        log = self.run_bzr("log")[0]
+        self.assertFalse('signature: no signature' in log)
+
+
 class TestLogVerbose(TestLog):
 
     def setUp(self):
@@ -463,6 +483,9 @@ class TestLogVerbose(TestLog):
     def test_log_short_verbose(self):
         self.assertUseShortDeltaFormat(['log', '--short', '-v'])
 
+    def test_log_s_verbose(self):
+        self.assertUseShortDeltaFormat(['log', '-S', '-v'])
+
     def test_log_short_verbose_verbose(self):
         self.assertUseLongDeltaFormat(['log', '--short', '-vv'])
 
@@ -924,3 +947,30 @@ class TestLogMultiple(TestLogWithLogCatcher):
         self.prepare_tree()
         os.chdir("dir1")
         self.assertLogRevnos(['dir2', 'file5'], ['5', '3'])
+
+
+class MainlineGhostTests(TestLogWithLogCatcher):
+
+    def setUp(self):
+        super(MainlineGhostTests, self).setUp()
+        tree = self.make_branch_and_tree('')
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        tree.add('')
+        tree.commit('msg1', rev_id='rev1')
+        tree.commit('msg2', rev_id='rev2')
+
+    def test_log_range(self):
+        self.assertLogRevnos(["-r1..2"], ["2", "1"])
+
+    def test_log_norange(self):
+        self.assertLogRevnos([], ["2", "1"])
+
+    def test_log_range_open_begin(self):
+        raise tests.KnownFailure("log with ghosts fails. bug #726466")
+        (stdout, stderr) = self.run_bzr(['log', '-r..2'], retcode=3)
+        self.assertEqual(["2", "1"],
+                         [r.revno for r in self.get_captured_revisions()])
+        self.assertEquals("bzr: ERROR: Further revision history missing.", stderr)
+
+    def test_log_range_open_end(self):
+        self.assertLogRevnos(["-r1.."], ["2", "1"])
