@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,24 +25,24 @@ from bzrlib import (
     urlutils,
     )
 from bzrlib.tests import EncodingAdapter
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
 
-def load_tests(standard_tests, module, loader):
-    return tests.multiply_tests(standard_tests,
-                                EncodingAdapter.encoding_scenarios,
-                                loader.suiteClass())
+load_tests = load_tests_apply_scenarios
 
 
 class TestNonAscii(tests.TestCaseWithTransport):
     """Test that bzr handles files/committers/etc which are non-ascii."""
 
+    scenarios = EncodingAdapter.encoding_scenarios
+
     def setUp(self):
         super(TestNonAscii, self).setUp()
         self._check_can_encode_paths()
 
         self.overrideAttr(osutils, '_cached_user_encoding', self.encoding)
         email = self.info['committer'] + ' <joe@foo.com>'
-        os.environ['BZR_EMAIL'] = email.encode(osutils.get_user_encoding())
+        self.overrideEnv('BZR_EMAIL', email.encode(osutils.get_user_encoding()))
         self.create_base()
 
     def run_bzr_decode(self, args, encoding=None, fail=False, retcode=None,
@@ -161,12 +161,12 @@ class TestNonAscii(tests.TestCaseWithTransport):
     def test_cat_revision(self):
         committer = self.info['committer']
         txt = self.run_bzr_decode('cat-revision -r 1')
-        self.failUnless(committer in txt,
+        self.assertTrue(committer in txt,
                         'failed to find %r in %r' % (committer, txt))
 
         msg = self.info['message']
         txt = self.run_bzr_decode('cat-revision -r 2')
-        self.failUnless(msg in txt, 'failed to find %r in %r' % (msg, txt))
+        self.assertTrue(msg in txt, 'failed to find %r in %r' % (msg, txt))
 
     def test_mkdir(self):
         txt = self.run_bzr_decode(['mkdir', self.info['directory']])
@@ -222,8 +222,8 @@ class TestNonAscii(tests.TestCaseWithTransport):
 
         txt = self.run_bzr_decode(['mv', 'a', fname2])
         self.assertEqual(u'a => %s\n' % fname2, txt)
-        self.failIfExists('a')
-        self.failUnlessExists(fname2)
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists(fname2)
 
         # After 'mv' we need to re-open the working tree
         self.wt = self.wt.bzrdir.open_workingtree()
@@ -240,7 +240,7 @@ class TestNonAscii(tests.TestCaseWithTransport):
         # The rename should still succeed
         newpath = u'%s/%s' % (dirname, fname2)
         txt = self.run_bzr_decode(['mv', newpath, 'a'], encoding='ascii')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.assertEqual(newpath.encode('ascii', 'replace') + ' => a\n', txt)
 
     def test_branch(self):
@@ -386,7 +386,7 @@ class TestNonAscii(tests.TestCaseWithTransport):
         self.assertEqual(fname+'\n', txt)
 
         txt = self.run_bzr_decode('deleted --show-ids')
-        self.failUnless(txt.startswith(fname))
+        self.assertTrue(txt.startswith(fname))
 
         # Deleted should fail if cannot decode
         # Because it is giving the exact paths
@@ -421,7 +421,7 @@ class TestNonAscii(tests.TestCaseWithTransport):
         self.wt.bzrdir.sprout(url)
 
         txt = self.run_bzr_decode('root', working_dir=dirname)
-        self.failUnless(txt.endswith(dirname+'\n'))
+        self.assertTrue(txt.endswith(dirname+'\n'))
 
         txt = self.run_bzr_decode('root', encoding='ascii', fail=True,
                                   working_dir=dirname)
