@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,28 +26,25 @@ from cStringIO import StringIO
 from StringIO import StringIO as pyStringIO
 import stat
 import sys
-import unittest
 
 from bzrlib import (
     errors,
     osutils,
+    pyutils,
     tests,
+    transport as _mod_transport,
     urlutils,
     )
 from bzrlib.errors import (ConnectionError,
-                           DirectoryNotEmpty,
                            FileExists,
                            InvalidURL,
-                           LockError,
                            NoSuchFile,
-                           NotLocalUrl,
                            PathError,
                            TransportNotPossible,
                            )
 from bzrlib.osutils import getcwd
 from bzrlib.smart import medium
 from bzrlib.tests import (
-    TestCaseInTempDir,
     TestSkipped,
     TestNotApplicable,
     multiply_tests,
@@ -56,7 +53,6 @@ from bzrlib.tests import test_server
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.transport import (
     ConnectedTransport,
-    get_transport,
     _get_transport_modules,
     )
 from bzrlib.transport.memory import MemoryTransport
@@ -78,7 +74,7 @@ def transport_test_permutations():
     for module in _get_transport_modules():
         try:
             permutations = get_transport_test_permutations(
-                reduce(getattr, (module).split('.')[1:], __import__(module)))
+                pyutils.get_named_object(module))
             for (klass, server_factory) in permutations:
                 scenario = ('%s,%s' % (klass.__name__, server_factory.__name__),
                     {"transport_class":klass,
@@ -102,11 +98,11 @@ class TransportTests(TestTransportImplementation):
 
     def setUp(self):
         super(TransportTests, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def check_transport_contents(self, content, transport, relpath):
-        """Check that transport.get(relpath).read() == content."""
-        self.assertEqualDiff(content, transport.get(relpath).read())
+        """Check that transport.get_bytes(relpath) == content."""
+        self.assertEqualDiff(content, transport.get_bytes(relpath))
 
     def test_ensure_base_missing(self):
         """.ensure_base() should create the directory if it doesn't exist"""
@@ -211,8 +207,17 @@ class TransportTests(TestTransportImplementation):
                     ]
         self.build_tree(files, transport=t, line_endings='binary')
         self.assertRaises(NoSuchFile, t.get, 'c')
-        self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
-        self.assertListRaises(NoSuchFile, t.get_multi, iter(['a', 'b', 'c']))
+        def iterate_and_close(func, *args):
+            for f in func(*args):
+                # We call f.read() here because things like paramiko actually
+                # spawn a thread to prefetch the content, which we want to
+                # consume before we close the handle.
+                content = f.read()
+                f.close()
+        self.assertRaises(NoSuchFile, iterate_and_close,
+                          t.get_multi, ['a', 'b', 'c'])
+        self.assertRaises(NoSuchFile, iterate_and_close,
+                          t.get_multi, iter(['a', 'b', 'c']))
 
     def test_get_directory_read_gives_ReadError(self):
         """consistent errors for read() on a file returned by get()."""
@@ -251,7 +256,6 @@ class TransportTests(TestTransportImplementation):
 
     def test_get_bytes_unknown_file(self):
         t = self.get_transport()
-
         self.assertRaises(NoSuchFile, t.get_bytes, 'c')
 
     def test_get_with_open_write_stream_sees_all_content(self):
@@ -261,7 +265,7 @@ class TransportTests(TestTransportImplementation):
         handle = t.open_write_stream('foo')
         try:
             handle.write('b')
-            self.assertEqual('b', t.get('foo').read())
+            self.assertEqual('b', t.get_bytes('foo'))
         finally:
             handle.close()
 
@@ -273,7 +277,11 @@ class TransportTests(TestTransportImplementation):
         try:
             handle.write('b')
             self.assertEqual('b', t.get_bytes('foo'))
-            self.assertEqual('b', t.get('foo').read())
+            f = t.get('foo')
+            try:
+                self.assertEqual('b', f.read())
+            finally:
+                f.close()
         finally:
             handle.close()
 
@@ -286,7 +294,7 @@ class TransportTests(TestTransportImplementation):
             return
 
         t.put_bytes('a', 'some text for a\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
 
         # The contents should be overwritten
@@ -304,9 +312,9 @@ class TransportTests(TestTransportImplementation):
                     t.put_bytes_non_atomic, 'a', 'some text for a\n')
             return
 
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
         t.put_bytes_non_atomic('a', 'some text for a\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         t.put_bytes_non_atomic('a', 'new\ncontents for\na\n')
@@ -324,7 +332,7 @@ class TransportTests(TestTransportImplementation):
         # Now test the create_parent flag
         self.assertRaises(NoSuchFile, t.put_bytes_non_atomic, 'dir/a',
                                        'contents\n')
-        self.failIf(t.has('dir/a'))
+        self.assertFalse(t.has('dir/a'))
         t.put_bytes_non_atomic('dir/a', 'contents for dir/a\n',
                                create_parent_dir=True)
         self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
@@ -402,7 +410,7 @@ class TransportTests(TestTransportImplementation):
         result = t.put_file('a', StringIO('some text for a\n'))
         # put_file returns the length of the data written
         self.assertEqual(16, result)
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         result = t.put_file('a', StringIO('new\ncontents for\na\n'))
@@ -420,9 +428,9 @@ class TransportTests(TestTransportImplementation):
                     t.put_file_non_atomic, 'a', StringIO('some text for a\n'))
             return
 
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
         t.put_file_non_atomic('a', StringIO('some text for a\n'))
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         t.put_file_non_atomic('a', StringIO('new\ncontents for\na\n'))
@@ -440,7 +448,7 @@ class TransportTests(TestTransportImplementation):
         # Now test the create_parent flag
         self.assertRaises(NoSuchFile, t.put_file_non_atomic, 'dir/a',
                                        StringIO('contents\n'))
-        self.failIf(t.has('dir/a'))
+        self.assertFalse(t.has('dir/a'))
         t.put_file_non_atomic('dir/a', StringIO('contents for dir/a\n'),
                               create_parent_dir=True)
         self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
@@ -645,7 +653,7 @@ class TransportTests(TestTransportImplementation):
             self.build_tree(files, transport=transport_from)
             self.assertEqual(4, transport_from.copy_to(files, transport_to))
             for f in files:
-                self.check_transport_contents(transport_to.get(f).read(),
+                self.check_transport_contents(transport_to.get_bytes(f),
                                               transport_from, f)
 
         t = self.get_transport()
@@ -674,7 +682,7 @@ class TransportTests(TestTransportImplementation):
         files = ['a', 'b', 'c', 'd']
         t.copy_to(iter(files), temp_transport)
         for f in files:
-            self.check_transport_contents(temp_transport.get(f).read(),
+            self.check_transport_contents(temp_transport.get_bytes(f),
                                           t, f)
         del temp_transport
 
@@ -823,9 +831,9 @@ class TransportTests(TestTransportImplementation):
             return
 
         t.put_bytes('a', 'a little bit of text\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         t.delete('a')
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
 
         self.assertRaises(NoSuchFile, t.delete, 'a')
 
@@ -837,9 +845,9 @@ class TransportTests(TestTransportImplementation):
         t.delete_multi(['a', 'c'])
         self.assertEqual([False, True, False],
                 list(t.has_multi(['a', 'b', 'c'])))
-        self.failIf(t.has('a'))
-        self.failUnless(t.has('b'))
-        self.failIf(t.has('c'))
+        self.assertFalse(t.has('a'))
+        self.assertTrue(t.has('b'))
+        self.assertFalse(t.has('c'))
 
         self.assertRaises(NoSuchFile,
                 t.delete_multi, ['a', 'b', 'c'])
@@ -906,7 +914,7 @@ class TransportTests(TestTransportImplementation):
         t.mkdir('foo-baz')
         t.rmdir('foo')
         self.assertRaises((NoSuchFile, PathError), t.rmdir, 'foo')
-        self.failUnless(t.has('foo-bar'))
+        self.assertTrue(t.has('foo-bar'))
 
     def test_rename_dir_succeeds(self):
         t = self.get_transport()
@@ -995,8 +1003,8 @@ class TransportTests(TestTransportImplementation):
         self.assertEquals([True, False], list(t.has_multi(['a', 'b'])))
 
         t.move('a', 'b')
-        self.failUnless(t.has('b'))
-        self.failIf(t.has('a'))
+        self.assertTrue(t.has('b'))
+        self.assertFalse(t.has('a'))
 
         self.check_transport_contents('a first file\n', t, 'b')
         self.assertEquals([False, True], list(t.has_multi(['a', 'b'])))
@@ -1004,7 +1012,7 @@ class TransportTests(TestTransportImplementation):
         # Overwrite a file
         t.put_bytes('c', 'c this file\n')
         t.move('c', 'b')
-        self.failIf(t.has('c'))
+        self.assertFalse(t.has('c'))
         self.check_transport_contents('c this file\n', t, 'b')
 
         # TODO: Try to write a test for atomicity
@@ -1042,7 +1050,7 @@ class TransportTests(TestTransportImplementation):
         except NotImplementedError:
             raise TestSkipped("Transport %s has no bogus URL support." %
                               self._server.__class__)
-        t = get_transport(url)
+        t = _mod_transport.get_transport(url)
         self.assertRaises((ConnectionError, NoSuchFile), t.get, '.bzr/branch')
 
     def test_stat(self):
@@ -1064,10 +1072,10 @@ class TransportTests(TestTransportImplementation):
         for path, size in zip(paths, sizes):
             st = t.stat(path)
             if path.endswith('/'):
-                self.failUnless(S_ISDIR(st.st_mode))
+                self.assertTrue(S_ISDIR(st.st_mode))
                 # directory sizes are meaningless
             else:
-                self.failUnless(S_ISREG(st.st_mode))
+                self.assertTrue(S_ISREG(st.st_mode))
                 self.assertEqual(size, st.st_size)
 
         remote_stats = list(t.stat_multi(paths))
@@ -1080,8 +1088,8 @@ class TransportTests(TestTransportImplementation):
         self.assertListRaises(NoSuchFile, t.stat_multi, iter(['a', 'c', 'd']))
         self.build_tree(['subdir/', 'subdir/file'], transport=t)
         subdir = t.clone('subdir')
-        subdir.stat('./file')
-        subdir.stat('.')
+        st = subdir.stat('./file')
+        st = subdir.stat('.')
 
     def test_hardlink(self):
         from stat import ST_NLINK
@@ -1096,11 +1104,11 @@ class TransportTests(TestTransportImplementation):
         try:
             t.hardlink(source_name, link_name)
 
-            self.failUnless(t.has(source_name))
-            self.failUnless(t.has(link_name))
+            self.assertTrue(t.has(source_name))
+            self.assertTrue(t.has(link_name))
 
             st = t.stat(link_name)
-            self.failUnlessEqual(st[ST_NLINK], 2)
+            self.assertEqual(st[ST_NLINK], 2)
         except TransportNotPossible:
             raise TestSkipped("Transport %s does not support hardlinks." %
                               self._server.__class__)
@@ -1118,11 +1126,12 @@ class TransportTests(TestTransportImplementation):
         try:
             t.symlink(source_name, link_name)
 
-            self.failUnless(t.has(source_name))
-            self.failUnless(t.has(link_name))
+            self.assertTrue(t.has(source_name))
+            self.assertTrue(t.has(link_name))
 
             st = t.stat(link_name)
-            self.failUnless(S_ISLNK(st.st_mode))
+            self.assertTrue(S_ISLNK(st.st_mode),
+                "expected symlink, got mode %o" % st.st_mode)
         except TransportNotPossible:
             raise TestSkipped("Transport %s does not support symlinks." %
                               self._server.__class__)
@@ -1265,7 +1274,7 @@ class TransportTests(TestTransportImplementation):
         self.assertIs(t._get_connection(), c._get_connection())
 
         # Temporary failure, we need to create a new dummy connection
-        new_connection = object()
+        new_connection = None
         t._set_connection(new_connection)
         # Check that both transports use the same connection
         self.assertIs(new_connection, t._get_connection())
@@ -1293,32 +1302,32 @@ class TransportTests(TestTransportImplementation):
 
         self.build_tree(['a', 'b/', 'b/c'], transport=t1)
 
-        self.failUnless(t1.has('a'))
-        self.failUnless(t1.has('b/c'))
-        self.failIf(t1.has('c'))
+        self.assertTrue(t1.has('a'))
+        self.assertTrue(t1.has('b/c'))
+        self.assertFalse(t1.has('c'))
 
         t2 = t1.clone('b')
         self.assertEqual(t1.base + 'b/', t2.base)
 
-        self.failUnless(t2.has('c'))
-        self.failIf(t2.has('a'))
+        self.assertTrue(t2.has('c'))
+        self.assertFalse(t2.has('a'))
 
         t3 = t2.clone('..')
-        self.failUnless(t3.has('a'))
-        self.failIf(t3.has('c'))
+        self.assertTrue(t3.has('a'))
+        self.assertFalse(t3.has('c'))
 
-        self.failIf(t1.has('b/d'))
-        self.failIf(t2.has('d'))
-        self.failIf(t3.has('b/d'))
+        self.assertFalse(t1.has('b/d'))
+        self.assertFalse(t2.has('d'))
+        self.assertFalse(t3.has('b/d'))
 
         if t1.is_readonly():
             self.build_tree_contents([('b/d', 'newfile\n')])
         else:
             t2.put_bytes('d', 'newfile\n')
 
-        self.failUnless(t1.has('b/d'))
-        self.failUnless(t2.has('d'))
-        self.failUnless(t3.has('b/d'))
+        self.assertTrue(t1.has('b/d'))
+        self.assertTrue(t2.has('d'))
+        self.assertTrue(t3.has('b/d'))
 
     def test_clone_to_root(self):
         orig_transport = self.get_transport()
@@ -1398,6 +1407,7 @@ class TransportTests(TestTransportImplementation):
         self.assertEqual(transport.clone("/").abspath('foo'),
                          transport.abspath("/foo"))
 
+    # GZ 2011-01-26: Test in per_transport but not using self.get_transport?
     def test_win32_abspath(self):
         # Note: we tried to set sys.platform='win32' so we could test on
         # other platforms too, but then osutils does platform specific
@@ -1408,12 +1418,12 @@ class TransportTests(TestTransportImplementation):
 
         # smoke test for abspath on win32.
         # a transport based on 'file:///' never fully qualifies the drive.
-        transport = get_transport("file:///")
-        self.failUnlessEqual(transport.abspath("/"), "file:///")
+        transport = _mod_transport.get_transport("file:///")
+        self.assertEqual(transport.abspath("/"), "file:///")
 
         # but a transport that starts with a drive spec must keep it.
-        transport = get_transport("file:///C:/")
-        self.failUnlessEqual(transport.abspath("/"), "file:///C:/")
+        transport = _mod_transport.get_transport("file:///C:/")
+        self.assertEqual(transport.abspath("/"), "file:///C:/")
 
     def test_local_abspath(self):
         transport = self.get_transport()
@@ -1616,7 +1626,7 @@ class TransportTests(TestTransportImplementation):
     def test_readv(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '0123456789')
 
@@ -1632,7 +1642,7 @@ class TransportTests(TestTransportImplementation):
     def test_readv_out_of_order(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '01234567890')
 
@@ -1710,7 +1720,7 @@ class TransportTests(TestTransportImplementation):
         transport = self.get_transport()
         # test from observed failure case.
         if transport.is_readonly():
-            file('a', 'w').write('a'*1024*1024)
+            with file('a', 'w') as f: f.write('a'*1024*1024)
         else:
             transport.put_bytes('a', 'a'*1024*1024)
         broken_vector = [(465219, 800), (225221, 800), (445548, 800),
@@ -1750,7 +1760,7 @@ class TransportTests(TestTransportImplementation):
     def test_readv_short_read(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '01234567890')
 
@@ -1765,3 +1775,15 @@ class TransportTests(TestTransportImplementation):
         # also raise a special error
         self.assertListRaises((errors.ShortReadvError, errors.InvalidRange),
                               transport.readv, 'a', [(12,2)])
+
+    def test_stat_symlink(self):
+        # if a transport points directly to a symlink (and supports symlinks
+        # at all) you can tell this.  helps with bug 32669.
+        t = self.get_transport()
+        try:
+            t.symlink('target', 'link')
+        except TransportNotPossible:
+            raise TestSkipped("symlinks not supported")
+        t2 = t.clone('link')
+        st = t2.stat('')
+        self.assertTrue(stat.S_ISLNK(st.st_mode))
