@@ -17,18 +17,15 @@
 """Tests for interface conformance of inventories of trees."""
 
 
-from cStringIO import StringIO
-import os
-
 from bzrlib import (
     tests,
     )
 from bzrlib.tests import (
-    features,
     per_tree,
     )
 from bzrlib.mutabletree import MutableTree
-from bzrlib.tests import SymlinkFeature, TestSkipped
+from bzrlib.tests import TestSkipped
+from bzrlib.tree import InventoryTree
 from bzrlib.transform import _PreviewTree
 from bzrlib.uncommit import uncommit
 
@@ -37,41 +34,6 @@ def get_entry(tree, file_id):
     return tree.iter_entries_by_dir([file_id]).next()[1]
 
 
-class TestPreviousHeads(per_tree.TestCaseWithTree):
-
-    def setUp(self):
-        # we want several inventories, that respectively
-        # give use the following scenarios:
-        # A) fileid not in any inventory (A),
-        # B) fileid present in one inventory (B) and (A,B)
-        # C) fileid present in two inventories, and they
-        #   are not mutual descendents (B, C)
-        # D) fileid present in two inventories and one is
-        #   a descendent of the other. (B, D)
-        super(TestPreviousHeads, self).setUp()
-        self.wt = self.make_branch_and_tree('.')
-        self.branch = self.wt.branch
-        self.build_tree(['file'])
-        self.wt.commit('new branch', allow_pointless=True, rev_id='A')
-        self.inv_A = self.branch.repository.get_inventory('A')
-        self.wt.add(['file'], ['fileid'])
-        self.wt.commit('add file', rev_id='B')
-        self.inv_B = self.branch.repository.get_inventory('B')
-        uncommit(self.branch, tree=self.wt)
-        self.assertEqual(self.branch.revision_history(), ['A'])
-        self.wt.commit('another add of file', rev_id='C')
-        self.inv_C = self.branch.repository.get_inventory('C')
-        self.wt.add_parent_tree_id('B')
-        self.wt.commit('merge in B', rev_id='D')
-        self.inv_D = self.branch.repository.get_inventory('D')
-        self.tree = self.workingtree_to_test_tree(self.wt)
-        self.tree.lock_read()
-        self.addCleanup(self.tree.unlock)
-        self.file_active = get_entry(self.tree, 'fileid')
-
-    # TODO: test two inventories with the same file revision
-
-
 class TestInventoryWithSymlinks(per_tree.TestCaseWithTree):
 
     _test_needs_features = [tests.SymlinkFeature]
@@ -140,32 +102,49 @@ class TestInventory(per_tree.TestCaseWithTree):
 
     def test_canonical_path(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir/file',
                          work_tree.get_canonical_inventory_path('Dir/File'))
 
     def test_canonical_path_before_commit(self):
         work_tree = self._make_canonical_test_tree(False)
-        # note: not committed.
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")        # note: not committed.
         self.assertEqual('dir/file',
                          work_tree.get_canonical_inventory_path('Dir/File'))
 
     def test_canonical_path_dir(self):
         # check it works when asked for just the directory portion.
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir', work_tree.get_canonical_inventory_path('Dir'))
 
     def test_canonical_path_root(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('', work_tree.get_canonical_inventory_path(''))
         self.assertEqual('/', work_tree.get_canonical_inventory_path('/'))
 
     def test_canonical_path_invalid_all(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('foo/bar',
                          work_tree.get_canonical_inventory_path('foo/bar'))
 
     def test_canonical_invalid_child(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir/None',
                          work_tree.get_canonical_inventory_path('Dir/None'))
 
@@ -180,6 +159,9 @@ class TestInventory(per_tree.TestCaseWithTree):
         work_tree.add(['test/', 'test/file', 'Test'])
 
         test_tree = self._convert_tree(work_tree)
+        if not isinstance(test_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         test_tree.lock_read()
         self.addCleanup(test_tree.unlock)
 
