@@ -17,18 +17,16 @@
 """Test that we can use smart_add on all Tree implementations."""
 
 from cStringIO import StringIO
+import os
 import sys
 
 from bzrlib import (
-    add,
     errors,
     ignores,
     osutils,
     tests,
-    workingtree,
     )
 from bzrlib.tests import (
-    features,
     test_smart_add,
     per_workingtree,
     )
@@ -202,6 +200,37 @@ class TestSmartAddTree(per_workingtree.TestCaseWithWorkingTree):
         self.assertEqual(['', 'dir', 'dir/subdir', 'dir/subdir/foo'],
             [path for path, ie in tree.iter_entries_by_dir()])
 
+    def test_add_dir_bug_251864(self):
+        """Added file turning into a dir should be detected on add dir
+
+        Similar to bug 205636 but with automatic adding of directory contents.
+        """
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir"])
+        tree.commit("Add dir contents")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
+    def test_add_subdir_file_bug_205636(self):
+        """Added file turning into a dir should be detected on add dir/file"""
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir/file"])
+        tree.commit("Add file in dir")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
     def test_custom_ids(self):
         sio = StringIO()
         action = test_smart_add.AddCustomIDAction(to_file=sio,
@@ -221,10 +250,10 @@ class TestSmartAddTree(per_workingtree.TestCaseWithWorkingTree):
         self.addCleanup(wt.unlock)
         self.assertEqual([('', wt.path2id('')),
                           ('dir1', 'directory-dir1'),
-                          ('dir1/file2', 'file-dir1%file2'),
                           ('file1', 'file-file1'),
+                          ('dir1/file2', 'file-dir1%file2'),
                          ], [(path, ie.file_id) for path, ie
-                                in wt.inventory.iter_entries()])
+                                in wt.iter_entries_by_dir()])
 
 
 class TestSmartAddConflictRelatedFiles(per_workingtree.TestCaseWithWorkingTree):
@@ -267,37 +296,44 @@ class TestSmartAddTreeUnicode(per_workingtree.TestCaseWithWorkingTree):
         self.wt = self.make_branch_and_tree('.')
         self.overrideAttr(osutils, 'normalized_filename')
 
-    def test_accessible_explicit(self):
+    def test_requires_normalized_unicode_filenames_fails_on_unnormalized(self):
+        """Adding unnormalized unicode filenames fail if and only if the
+        workingtree format has the requires_normalized_unicode_filenames flag
+        set and the underlying filesystem doesn't normalize.
+        """
         osutils.normalized_filename = osutils._accessible_normalized_filename
-        if isinstance(self.workingtree_format, workingtree.WorkingTreeFormat2):
-            self.expectFailure(
-                'With WorkingTreeFormat2, smart_add requires'
-                ' normalized unicode filenames',
-                self.assertRaises, errors.NoSuchFile,
-                self.wt.smart_add, [u'a\u030a'])
+        if (self.workingtree_format.requires_normalized_unicode_filenames
+            and sys.platform != 'darwin'):
+            self.assertRaises(
+                errors.NoSuchFile, self.wt.smart_add, [u'a\u030a'])
         else:
             self.wt.smart_add([u'a\u030a'])
+
+    def test_accessible_explicit(self):
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        if self.workingtree_format.requires_normalized_unicode_filenames:
+            raise tests.TestNotApplicable(
+                'Working tree format smart_add requires normalized unicode '
+                'filenames')
+        self.wt.smart_add([u'a\u030a'])
         self.wt.lock_read()
         self.addCleanup(self.wt.unlock)
         self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
                          [(path, ie.kind) for path,ie in
-                          self.wt.inventory.iter_entries()])
+                          self.wt.iter_entries_by_dir()])
 
     def test_accessible_implicit(self):
         osutils.normalized_filename = osutils._accessible_normalized_filename
-        if isinstance(self.workingtree_format, workingtree.WorkingTreeFormat2):
-            self.expectFailure(
-                'With WorkingTreeFormat2, smart_add requires'
-                ' normalized unicode filenames',
-                self.assertRaises, errors.NoSuchFile,
-                self.wt.smart_add, [])
-        else:
-            self.wt.smart_add([])
+        if self.workingtree_format.requires_normalized_unicode_filenames:
+            raise tests.TestNotApplicable(
+                'Working tree format smart_add requires normalized unicode '
+                'filenames')
+        self.wt.smart_add([])
         self.wt.lock_read()
         self.addCleanup(self.wt.unlock)
         self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
                          [(path, ie.kind) for path,ie
-                          in self.wt.inventory.iter_entries()])
+                          in self.wt.iter_entries_by_dir()])
 
     def test_inaccessible_explicit(self):
         osutils.normalized_filename = osutils._inaccessible_normalized_filename
