@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,66 +28,34 @@ from bzrlib import (
     )
 from bzrlib.tests import (
     test_dirstate,
-    test_osutils,
+    )
+from bzrlib.tests.test_osutils import dir_reader_scenarios
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    multiply_scenarios,
     )
 
-try:
-    from bzrlib import _dirstate_helpers_pyx
-    has_dirstate_helpers_pyx = True
-except ImportError:
-    has_dirstate_helpers_pyx = False
+
+load_tests = load_tests_apply_scenarios
 
 
-class _CompiledDirstateHelpersFeature(tests.Feature):
-    def _probe(self):
-        return has_dirstate_helpers_pyx
-
-    def feature_name(self):
-        return 'bzrlib._dirstate_helpers_pyx'
-CompiledDirstateHelpersFeature = _CompiledDirstateHelpersFeature()
+compiled_dirstate_helpers_feature = tests.ModuleAvailableFeature(
+    'bzrlib._dirstate_helpers_pyx')
 
 
-def load_tests(basic_tests, module, loader):
-    # FIXME: we should also parametrize against SHA1Provider !
-    suite = loader.suiteClass()
-    remaining_tests = basic_tests
+# FIXME: we should also parametrize against SHA1Provider !
 
-    dir_reader_scenarios = test_osutils.dir_reader_scenarios()
+ue_scenarios = [('dirstate_Python',
+    {'update_entry': dirstate.py_update_entry})]
+if compiled_dirstate_helpers_feature.available():
+    update_entry = compiled_dirstate_helpers_feature.module.update_entry
+    ue_scenarios.append(('dirstate_Pyrex', {'update_entry': update_entry}))
 
-    ue_scenarios = [('dirstate_Python',
-                     {'update_entry': dirstate.py_update_entry})]
-    if has_dirstate_helpers_pyx:
-        pyrex_scenario = ('dirstate_Pyrex',
-                          {'update_entry': _dirstate_helpers_pyx.update_entry})
-        ue_scenarios.append(pyrex_scenario)
-    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(TestUpdateEntry))
-    tests.multiply_tests(process_entry_tests,
-                         tests.multiply_scenarios(dir_reader_scenarios,
-                                                  ue_scenarios),
-                         suite)
-
-    pe_scenarios = [('dirstate_Python',
-                     {'_process_entry': dirstate.ProcessEntryPython})]
-    if has_dirstate_helpers_pyx:
-        pyrex_scenario = (
-            'dirstate_Pyrex',
-            {'_process_entry': _dirstate_helpers_pyx.ProcessEntryC})
-        pe_scenarios.append(pyrex_scenario)
-    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(TestProcessEntry))
-    tests.multiply_tests(process_entry_tests,
-                         tests.multiply_scenarios(dir_reader_scenarios,
-                                                  pe_scenarios),
-                         suite)
-
-    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(
-            test_dirstate.TestCaseWithDirState))
-    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios, suite)
-    suite.addTest(remaining_tests)
-
-    return suite
+pe_scenarios = [('dirstate_Python',
+    {'_process_entry': dirstate.ProcessEntryPython})]
+if compiled_dirstate_helpers_feature.available():
+    process_entry = compiled_dirstate_helpers_feature.module.ProcessEntryC
+    pe_scenarios.append(('dirstate_Pyrex', {'_process_entry': process_entry}))
 
 
 class TestBisectPathMixin(object):
@@ -259,7 +227,7 @@ class TestBisectPathLeft(tests.TestCase, TestBisectPathMixin):
 class TestCompiledBisectPathLeft(TestBisectPathLeft):
     """Run all Bisect Path tests against _bisect_path_lect"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_bisect_path(self):
         from bzrlib._dirstate_helpers_pyx import _bisect_path_left
@@ -280,7 +248,7 @@ class TestBisectPathRight(tests.TestCase, TestBisectPathMixin):
 class TestCompiledBisectPathRight(TestBisectPathRight):
     """Run all Bisect Path tests against _bisect_path_right"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_bisect_path(self):
         from bzrlib._dirstate_helpers_pyx import _bisect_path_right
@@ -392,7 +360,7 @@ class TestCompiledBisectDirblock(TestBisectDirblock):
     compiled version.
     """
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_bisect_dirblock(self):
         from bzrlib._dirstate_helpers_pyx import bisect_dirblock
@@ -514,7 +482,7 @@ class TestCmpByDirs(tests.TestCase):
 class TestCompiledCmpByDirs(TestCmpByDirs):
     """Test the pyrex implementation of cmp_by_dirs"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_cmp_by_dirs(self):
         from bzrlib._dirstate_helpers_pyx import cmp_by_dirs
@@ -665,7 +633,7 @@ class TestCmpPathByDirblock(tests.TestCase):
 class TestCompiledCmpPathByDirblock(TestCmpPathByDirblock):
     """Test the pyrex implementation of _cmp_path_by_dirblock"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_cmp_by_dirs(self):
         from bzrlib._dirstate_helpers_pyx import _cmp_path_by_dirblock
@@ -675,7 +643,7 @@ class TestCompiledCmpPathByDirblock(TestCmpPathByDirblock):
 class TestMemRChr(tests.TestCase):
     """Test memrchr functionality"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def assertMemRChr(self, expected, s, c):
         from bzrlib._dirstate_helpers_pyx import _py_memrchr
@@ -725,6 +693,8 @@ class TestReadDirblocks(test_dirstate.TestCaseWithDirState):
     implementation.
     """
 
+    # inherits scenarios from test_dirstate
+
     def get_read_dirblocks(self):
         from bzrlib._dirstate_helpers_py import _read_dirblocks
         return _read_dirblocks
@@ -743,13 +713,17 @@ class TestReadDirblocks(test_dirstate.TestCaseWithDirState):
 
     def test_trailing_garbage(self):
         tree, state, expected = self.create_basic_dirstate()
-        # We can modify the file as long as it hasn't been read yet.
+        # On Unix, we can write extra data as long as we haven't read yet, but
+        # on Win32, if you've opened the file with FILE_SHARE_READ, trying to
+        # open it in append mode will fail.
+        state.unlock()
         f = open('dirstate', 'ab')
         try:
             # Add bogus trailing garbage
             f.write('bogus\n')
         finally:
             f.close()
+            state.lock_read()
         e = self.assertRaises(errors.DirstateCorrupt,
                               state._read_dirblocks_if_needed)
         # Make sure we mention the bogus characters in the error
@@ -759,7 +733,7 @@ class TestReadDirblocks(test_dirstate.TestCaseWithDirState):
 class TestCompiledReadDirblocks(TestReadDirblocks):
     """Test the pyrex implementation of _read_dirblocks"""
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    _test_needs_features = [compiled_dirstate_helpers_feature]
 
     def get_read_dirblocks(self):
         from bzrlib._dirstate_helpers_pyx import _read_dirblocks
@@ -775,49 +749,49 @@ class TestUsingCompiledIfAvailable(tests.TestCase):
     """
 
     def test_bisect_dirblock(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import bisect_dirblock
         else:
             from bzrlib._dirstate_helpers_py import bisect_dirblock
         self.assertIs(bisect_dirblock, dirstate.bisect_dirblock)
 
     def test__bisect_path_left(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import _bisect_path_left
         else:
             from bzrlib._dirstate_helpers_py import _bisect_path_left
         self.assertIs(_bisect_path_left, dirstate._bisect_path_left)
 
     def test__bisect_path_right(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import _bisect_path_right
         else:
             from bzrlib._dirstate_helpers_py import _bisect_path_right
         self.assertIs(_bisect_path_right, dirstate._bisect_path_right)
 
     def test_cmp_by_dirs(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import cmp_by_dirs
         else:
             from bzrlib._dirstate_helpers_py import cmp_by_dirs
         self.assertIs(cmp_by_dirs, dirstate.cmp_by_dirs)
 
     def test__read_dirblocks(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import _read_dirblocks
         else:
             from bzrlib._dirstate_helpers_py import _read_dirblocks
         self.assertIs(_read_dirblocks, dirstate._read_dirblocks)
 
     def test_update_entry(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import update_entry
         else:
             from bzrlib.dirstate import update_entry
         self.assertIs(update_entry, dirstate.update_entry)
 
     def test_process_entry(self):
-        if CompiledDirstateHelpersFeature.available():
+        if compiled_dirstate_helpers_feature.available():
             from bzrlib._dirstate_helpers_pyx import ProcessEntryC
             self.assertIs(ProcessEntryC, dirstate._process_entry)
         else:
@@ -828,16 +802,15 @@ class TestUsingCompiledIfAvailable(tests.TestCase):
 class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
     """Test the DirState.update_entry functions"""
 
+    scenarios = multiply_scenarios(
+        dir_reader_scenarios(), ue_scenarios)
+
     # Set by load_tests
     update_entry = None
 
     def setUp(self):
         super(TestUpdateEntry, self).setUp()
-        orig = dirstate.update_entry
-        def cleanup():
-            dirstate.update_entry = orig
-        self.addCleanup(cleanup)
-        dirstate.update_entry = self.update_entry
+        self.overrideAttr(dirstate, 'update_entry', self.update_entry)
 
     def get_state_with_a(self):
         """Create a DirState tracking a single object named 'a'"""
@@ -849,25 +822,34 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_observed_sha1_cachable(self):
         state, entry = self.get_state_with_a()
+        state.save()
         atime = time.time() - 10
         self.build_tree(['a'])
-        statvalue = os.lstat('a')
-        statvalue = test_dirstate._FakeStat(statvalue.st_size, atime, atime,
-            statvalue.st_dev, statvalue.st_ino, statvalue.st_mode)
+        statvalue = test_dirstate._FakeStat.from_stat(os.lstat('a'))
+        statvalue.st_mtime = statvalue.st_ctime = atime
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         state._observed_sha1(entry, "foo", statvalue)
         self.assertEqual('foo', entry[1][0][1])
         packed_stat = dirstate.pack_stat(statvalue)
         self.assertEqual(packed_stat, entry[1][0][4])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
+                         state._dirblock_state)
 
     def test_observed_sha1_not_cachable(self):
         state, entry = self.get_state_with_a()
+        state.save()
         oldval = entry[1][0][1]
         oldstat = entry[1][0][4]
         self.build_tree(['a'])
         statvalue = os.lstat('a')
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         state._observed_sha1(entry, "foo", statvalue)
         self.assertEqual(oldval, entry[1][0][1])
         self.assertEqual(oldstat, entry[1][0][4])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
 
     def test_update_entry(self):
         state, _ = self.get_state_with_a()
@@ -898,10 +880,12 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
                                           stat_value=stat_value)
         self.assertEqual(None, link_or_sha1)
 
-        # The dirblock entry should not have cached the file's sha1 (too new)
+        # The dirblock entry should not have computed or cached the file's
+        # sha1, but it did update the files' st_size. However, this is not
+        # worth writing a dirstate file for, so we leave the state UNMODIFIED
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
         mode = stat_value.st_mode
         self.assertEqual([('is_exec', mode, False)], state._log)
@@ -910,9 +894,8 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
 
-        # If we do it again right away, we don't know if the file has changed
-        # so we will re-read the file. Roll the clock back so the file is
-        # guaranteed to look too new.
+        # Roll the clock back so the file is guaranteed to look too new. We
+        # should still not compute the sha1.
         state.adjust_time(-10)
         del state._log[:]
 
@@ -920,7 +903,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
                                           stat_value=stat_value)
         self.assertEqual([('is_exec', mode, False)], state._log)
         self.assertEqual(None, link_or_sha1)
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
@@ -936,6 +919,8 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         self.assertEqual([('is_exec', mode, False)], state._log)
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
 
         # If the file is no longer new, and the clock has been moved forward
         # sufficiently, it will cache the sha.
@@ -983,35 +968,38 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         # Dirblock is not updated (the link is too new)
         self.assertEqual([('l', '', 6, False, dirstate.DirState.NULLSTAT)],
                          entry[1])
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        # The file entry turned into a symlink, that is considered
+        # HASH modified worthy.
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
                          state._dirblock_state)
 
         # Because the stat_value looks new, we should re-read the target
+        del state._log[:]
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
         self.assertEqual('target', link_or_sha1)
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([('read_link', 'a', '')], state._log)
         self.assertEqual([('l', '', 6, False, dirstate.DirState.NULLSTAT)],
                          entry[1])
+        state.save()
         state.adjust_time(+20) # Skip into the future, all files look old
+        del state._log[:]
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
+        # The symlink stayed a symlink. So while it is new enough to cache, we
+        # don't bother setting the flag, because it is not really worth saving
+        # (when we stat the symlink, we'll have paged in the target.)
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         self.assertEqual('target', link_or_sha1)
         # We need to re-read the link because only now can we cache it
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([('read_link', 'a', '')], state._log)
         self.assertEqual([('l', 'target', 6, False, packed_stat)],
                          entry[1])
 
+        del state._log[:]
         # Another call won't re-read the link
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([], state._log)
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
         self.assertEqual('target', link_or_sha1)
@@ -1032,14 +1020,31 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         self.build_tree(['a/'])
         state.adjust_time(+20)
         self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        # a/ used to be a file, but is now a directory, worth saving
         self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
                          state._dirblock_state)
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
+        # No changes to a/ means not worth saving.
         self.assertIs(None, self.do_update_entry(state, entry, 'a'))
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
+        # Change the last-modified time for the directory
+        t = time.time() - 100.0
+        try:
+            os.utime('a', (t, t))
+        except OSError:
+            # It looks like Win32 + FAT doesn't allow to change times on a dir.
+            raise tests.TestSkipped("can't update mtime of a dir on FAT")
+        saved_packed_stat = entry[1][0][-1]
+        self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        # We *do* go ahead and update the information in the dirblocks, but we
+        # don't bother setting IN_MEMORY_MODIFIED because it is trivial to
+        # recompute.
+        self.assertNotEqual(saved_packed_stat, entry[1][0][-1])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
 
     def test_update_entry_file_unchanged(self):
         state, _ = self.get_state_with_a()
@@ -1275,16 +1280,14 @@ class UppercaseSHA1Provider(dirstate.SHA1Provider):
 
 class TestProcessEntry(test_dirstate.TestCaseWithDirState):
 
+    scenarios = multiply_scenarios(dir_reader_scenarios(), pe_scenarios)
+
     # Set by load_tests
     _process_entry = None
 
     def setUp(self):
         super(TestProcessEntry, self).setUp()
-        orig = dirstate._process_entry
-        def cleanup():
-            dirstate._process_entry = orig
-        self.addCleanup(cleanup)
-        dirstate._process_entry = self._process_entry
+        self.overrideAttr(dirstate, '_process_entry', self._process_entry)
 
     def assertChangedFileIds(self, expected, tree):
         tree.lock_read()
@@ -1295,6 +1298,23 @@ class TestProcessEntry(test_dirstate.TestCaseWithDirState):
             tree.unlock()
         self.assertEqual(sorted(expected), sorted(file_ids))
 
+    def test_exceptions_raised(self):
+        # This is a direct test of bug #495023, it relies on osutils.is_inside
+        # getting called in an inner function. Which makes it a bit brittle,
+        # but at least it does reproduce the bug.
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file', 'tree/dir/', 'tree/dir/sub',
+                         'tree/dir2/', 'tree/dir2/sub2'])
+        tree.add(['file', 'dir', 'dir/sub', 'dir2', 'dir2/sub2'])
+        tree.commit('first commit')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        basis_tree = tree.basis_tree()
+        def is_inside_raises(*args, **kwargs):
+            raise RuntimeError('stop this')
+        self.overrideAttr(osutils, 'is_inside', is_inside_raises)
+        self.assertListRaises(RuntimeError, tree.iter_changes, basis_tree)
+
     def test_simple_changes(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/file'])
@@ -1314,3 +1334,56 @@ class TestProcessEntry(test_dirstate.TestCaseWithDirState):
         state._sha1_provider = UppercaseSHA1Provider()
         self.assertChangedFileIds(['file-id'], tree)
 
+
+class TestPackStat(tests.TestCase):
+    """Check packed representaton of stat values is robust on all inputs"""
+
+    # GZ 2011-09-26: Should parametrise against all pack_stat implementations
+
+    @staticmethod
+    def pack(statlike_tuple):
+        return dirstate.pack_stat(os.stat_result(statlike_tuple))
+
+    @staticmethod
+    def unpack_field(packed_string, stat_field):
+        return dirstate._unpack_stat(packed_string)[stat_field]
+
+    def test_result(self):
+        self.assertEqual("AAAQAAAAABAAAAARAAAAAgAAAAEAAIHk",
+            self.pack((33252, 1, 2, 0, 0, 0, 4096, 15.5, 16.5, 17.5)))
+
+    def test_giant_inode(self):
+        packed = self.pack((33252, 0xF80000ABC, 0, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x80000ABC, self.unpack_field(packed, "st_ino"))
+
+    def test_giant_size(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, (1 << 33) + 4096, 0, 0, 0))
+        self.assertEqual(4096, self.unpack_field(packed, "st_size"))
+
+    def test_fractional_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 16.9375, 0))
+        self.assertEqual(16, self.unpack_field(packed, "st_mtime"))
+
+    def test_ancient_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, -11644473600.0, 0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_mtime"))
+
+    def test_distant_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 64060588800.0, 0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_mtime"))
+
+    def test_fractional_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 17.5625))
+        self.assertEqual(17, self.unpack_field(packed, "st_ctime"))
+
+    def test_ancient_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, -11644473600.0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_ctime"))
+
+    def test_distant_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 64060588800.0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_ctime"))
+
+    def test_negative_dev(self):
+        packed = self.pack((33252, 0, -0xFFFFFCDE, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x322, self.unpack_field(packed, "st_dev"))
