@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    errors,
     revision as _mod_revision,
     tests,
     )
@@ -248,6 +247,18 @@ class TestBranchBuilderBuildSnapshot(tests.TestCaseWithMemoryTransport):
                               (u'dir', 'dir-id', 'directory'),
                               (u'dir/a', 'a-id', 'file')], rev_tree)
 
+    def test_rename_out_of_unversioned_subdir(self):
+        builder = self.build_a_rev()
+        builder.build_snapshot('B-id', None,
+            [('add', ('dir', 'dir-id', 'directory', None)),
+             ('rename', ('a', 'dir/a'))])
+        builder.build_snapshot('C-id', None,
+            [('rename', ('dir/a', 'a')),
+             ('unversion', 'dir-id')])
+        rev_tree = builder.get_branch().repository.revision_tree('C-id')
+        self.assertTreeShape([(u'', 'a-root-id', 'directory'),
+                              (u'a', 'a-id', 'file')], rev_tree)
+
     def test_set_parent(self):
         builder = self.build_a_rev()
         builder.start_series()
@@ -322,8 +333,23 @@ class TestBranchBuilderBuildSnapshot(tests.TestCaseWithMemoryTransport):
                              ], d_tree)
         # Because we copied the exact text into *this* tree, the 'c' file
         # should look like it was not modified in the merge
-        self.assertEqual('C-id', d_tree.inventory['c-id'].revision)
+        self.assertEqual('C-id', d_tree.get_file_revision('c-id'))
 
+    def test_set_parent_to_null(self):
+        builder = self.build_a_rev()
+        builder.start_series()
+        self.addCleanup(builder.finish_series)
+        builder.build_snapshot('B-id', [],
+            [('add', ('', None, 'directory', None))])
+        # We should now have a graph:
+        #   A B
+        # And not A => B
+        repo = builder.get_branch().repository
+        self.assertEqual({'A-id': (_mod_revision.NULL_REVISION,),
+                          'B-id': (_mod_revision.NULL_REVISION,),},
+                         repo.get_parent_map(['A-id', 'B-id']))
+
+    
     def test_start_finish_series(self):
         builder = BranchBuilder(self.get_transport().clone('foo'))
         builder.start_series()
@@ -350,3 +376,63 @@ class TestBranchBuilderBuildSnapshot(tests.TestCaseWithMemoryTransport):
         self.addCleanup(b.unlock)
         self.assertEqual(('ghost',),
             b.repository.get_graph().get_parent_map(['tip'])['tip'])
+
+    def test_unversion_root_add_new_root(self):
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('rev-1', None,
+            [('add', ('', 'TREE_ROOT', 'directory', ''))])
+        builder.build_snapshot('rev-2', None,
+            [('unversion', 'TREE_ROOT'),
+             ('add', ('', 'my-root', 'directory', ''))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('rev-2')
+        self.assertTreeShape([(u'', 'my-root', 'directory')], rev_tree)
+
+    def test_empty_flush(self):
+        """A flush with no actions before it is a no-op."""
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('rev-1', None,
+            [('add', ('', 'TREE_ROOT', 'directory', ''))])
+        builder.build_snapshot('rev-2', None, [('flush', None)])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('rev-2')
+        self.assertTreeShape([(u'', 'TREE_ROOT', 'directory')], rev_tree)
+
+    def test_kind_change(self):
+        """It's possible to change the kind of an entry in a single snapshot
+        with a bit of help from the 'flush' action.
+        """
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('A-id', None,
+            [('add', (u'', 'a-root-id', 'directory', None)),
+             ('add', (u'a', 'a-id', 'file', 'content\n'))])
+        builder.build_snapshot('B-id', None,
+            [('unversion', 'a-id'),
+             ('flush', None),
+             ('add', (u'a', 'a-id', 'directory', None))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('B-id')
+        self.assertTreeShape(
+            [(u'', 'a-root-id', 'directory'), (u'a', 'a-id', 'directory')],
+            rev_tree)
+
+    def test_pivot_root(self):
+        """It's possible (albeit awkward) to move an existing dir to the root
+        in a single snapshot by using unversion then flush then add.
+        """
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('A-id', None,
+            [('add', (u'', 'orig-root', 'directory', None)),
+             ('add', (u'dir', 'dir-id', 'directory', None))])
+        builder.build_snapshot('B-id', None,
+            [('unversion', 'orig-root'),  # implicitly unversions all children
+             ('flush', None),
+             ('add', (u'', 'dir-id', 'directory', None))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('B-id')
+        self.assertTreeShape([(u'', 'dir-id', 'directory')], rev_tree)
+
