@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Test the lazy_import functionality."""
 
@@ -24,7 +24,10 @@ from bzrlib import (
     lazy_import,
     osutils,
     )
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    )
 
 
 class InstrumentedReplacer(lazy_import.ScopeReplacer):
@@ -107,7 +110,7 @@ class TestScopeReplacer(TestCase):
 
     def test_object(self):
         """ScopeReplacer can create an instance in local scope.
-        
+
         An object should appear in globals() by constructing a ScopeReplacer,
         and it will be replaced with the real object upon the first request.
         """
@@ -127,13 +130,9 @@ class TestScopeReplacer(TestCase):
         else:
             self.fail('test_obj1 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         InstrumentedReplacer(scope=globals(), name='test_obj1',
                              factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj1.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -156,6 +155,8 @@ class TestScopeReplacer(TestCase):
         An object should appear in globals() by constructing a ScopeReplacer,
         and it will be replaced with the real object upon the first request.
         """
+        actions = []
+        TestClass.use_actions(actions)
         def factory(replacer, scope, name):
             return TestClass()
         try:
@@ -166,13 +167,9 @@ class TestScopeReplacer(TestCase):
         else:
             self.fail('test_obj6 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         lazy_import.ScopeReplacer(scope=globals(), name='test_obj6',
                                   factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj6.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -268,7 +265,7 @@ class TestScopeReplacer(TestCase):
         InstrumentedReplacer(scope=globals(), name='test_class2',
                              factory=factory)
 
-        self.failIf(test_class2 is TestClass)
+        self.assertFalse(test_class2 is TestClass)
         obj = test_class2()
         self.assertIs(test_class2, TestClass)
         self.assertIsInstance(obj, TestClass)
@@ -301,7 +298,7 @@ class TestScopeReplacer(TestCase):
         InstrumentedReplacer(scope=globals(), name='test_func1',
                              factory=factory)
 
-        self.failIf(test_func1 is func)
+        self.assertFalse(test_func1 is func)
         val = test_func1(1, 2, c='3')
         self.assertIs(test_func1, func)
 
@@ -349,7 +346,7 @@ class TestScopeReplacer(TestCase):
                          object.__getattribute__(test_obj2, '__class__'))
         self.assertEqual(InstrumentedReplacer,
                          object.__getattribute__(test_obj3, '__class__'))
-        
+
         # The first use of the alternate variable causes test_obj2 to
         # be replaced.
         self.assertEqual('foo', test_obj3.foo(1))
@@ -367,7 +364,7 @@ class TestScopeReplacer(TestCase):
         # because only now are we able to detect the problem.
         self.assertRaises(errors.IllegalUseOfScopeReplacer,
                           getattr, test_obj3, 'foo')
-        
+
         self.assertEqual([('__getattribute__', 'foo'),
                           '_replace',
                           'factory',
@@ -438,6 +435,36 @@ class TestScopeReplacer(TestCase):
                           ('foo', 4),
                          ], actions)
 
+    def test_replacing_from_own_scope_fails(self):
+        """If a ScopeReplacer tries to replace itself a nice error is given"""
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            # return the name in given scope, which is currently the replacer
+            return scope[name]
+
+        try:
+            test_obj7
+        except NameError:
+            # test_obj7 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj7 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_obj7',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj7, '__class__'))
+        e = self.assertRaises(errors.IllegalUseOfScopeReplacer, test_obj7)
+        self.assertIn("replace itself", e.msg)
+        self.assertEqual([('__call__', (), {}),
+                          '_replace',
+                          'factory'], actions)
+
 
 class ImportReplacerHelper(TestCaseInTempDir):
     """Test the ability to have a lazily imported module or object"""
@@ -450,17 +477,16 @@ class ImportReplacerHelper(TestCaseInTempDir):
         self.actions = []
         InstrumentedImportReplacer.use_actions(self.actions)
 
+        sys.path.append(base_path)
+        self.addCleanup(sys.path.remove, base_path)
+
         original_import = __import__
         def instrumented_import(mod, scope1, scope2, fromlist):
             self.actions.append(('import', mod, fromlist))
             return original_import(mod, scope1, scope2, fromlist)
-
         def cleanup():
-            if base_path in sys.path:
-                sys.path.remove(base_path)
             __builtins__['__import__'] = original_import
         self.addCleanup(cleanup)
-        sys.path.append(base_path)
         __builtins__['__import__'] = instrumented_import
 
     def create_modules(self):
