@@ -117,6 +117,7 @@ class TestCaseForLogFormatter(tests.TestCaseWithTransport, TestLogMixin):
             branch.tags.set_tag('v1.0', 'rev-3')
         return wt
 
+
 class LogCatcher(log.LogFormatter):
     """Pull log messages into a list rather than displaying them.
 
@@ -371,6 +372,32 @@ Use --include-merges or -n0 to see merged revisions.
             wt.branch, log.ShortLogFormatter,
             show_log_kwargs=dict(start_revision=rev, end_revision=rev))
 
+    def test_show_ids(self):
+        wt = self.make_branch_and_tree('parent')
+        self.build_tree(['parent/f1', 'parent/f2'])
+        wt.add(['f1','f2'])
+        self.wt_commit(wt, 'first post', rev_id='a')
+        child_wt = wt.bzrdir.sprout('child').open_workingtree()
+        self.wt_commit(child_wt, 'branch 1 changes', rev_id='b')
+        wt.merge_from_branch(child_wt.branch)
+        self.wt_commit(wt, 'merge branch 1', rev_id='c')
+        self.assertFormatterResult("""\
+    2 Joe Foo\t2005-11-22 [merge]
+      revision-id:c
+      merge branch 1
+
+          1.1.1 Joe Foo\t2005-11-22
+                revision-id:b
+                branch 1 changes
+
+    1 Joe Foo\t2005-11-22
+      revision-id:a
+      first post
+
+""",
+            wt.branch, log.ShortLogFormatter,
+            formatter_kwargs=dict(levels=0,show_ids=True))
+
 
 class TestShortLogFormatterWithMergeRevisions(TestCaseForLogFormatter):
 
@@ -659,6 +686,47 @@ message:
         self.assertEqualDiff('''custom_prop_name: test_value\n''',
                              sio.getvalue())
 
+    def test_show_ids(self):
+        wt = self.make_branch_and_tree('parent')
+        self.build_tree(['parent/f1', 'parent/f2'])
+        wt.add(['f1','f2'])
+        self.wt_commit(wt, 'first post', rev_id='a')
+        child_wt = wt.bzrdir.sprout('child').open_workingtree()
+        self.wt_commit(child_wt, 'branch 1 changes', rev_id='b')
+        wt.merge_from_branch(child_wt.branch)
+        self.wt_commit(wt, 'merge branch 1', rev_id='c')
+        self.assertFormatterResult("""\
+------------------------------------------------------------
+revno: 2 [merge]
+revision-id: c
+parent: a
+parent: b
+committer: Joe Foo <joe@foo.com>
+branch nick: parent
+timestamp: Tue 2005-11-22 00:00:02 +0000
+message:
+  merge branch 1
+    ------------------------------------------------------------
+    revno: 1.1.1
+    revision-id: b
+    parent: a
+    committer: Joe Foo <joe@foo.com>
+    branch nick: child
+    timestamp: Tue 2005-11-22 00:00:01 +0000
+    message:
+      branch 1 changes
+------------------------------------------------------------
+revno: 1
+revision-id: a
+committer: Joe Foo <joe@foo.com>
+branch nick: parent
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  first post
+""",
+            wt.branch, log.LongLogFormatter,
+            formatter_kwargs=dict(levels=0,show_ids=True))
+
 
 class TestLongLogFormatterWithoutMergeRevisions(TestCaseForLogFormatter):
 
@@ -1463,6 +1531,124 @@ class TestHistoryChange(tests.TestCaseWithTransport):
         self.assertNotContainsRe(s.getvalue(), 'Added Revisions:')
 
 
+class TestRevisionNotInBranch(TestCaseForLogFormatter):
+
+    def setup_a_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        kwargs = {
+            'committer': 'Joe Foo <joe@foo.com>',
+            'timestamp': 1132617600, # Mon 2005-11-22 00:00:00 +0000
+            'timezone': 0, # UTC
+        }
+        tree.commit('commit 1a', rev_id='1a', **kwargs)
+        tree.commit('commit 2a', rev_id='2a', **kwargs)
+        tree.commit('commit 3a', rev_id='3a', **kwargs)
+        return tree
+
+    def setup_ab_tree(self):
+        tree = self.setup_a_tree()
+        tree.set_last_revision('1a')
+        tree.branch.set_last_revision_info(1, '1a')
+        kwargs = {
+            'committer': 'Joe Foo <joe@foo.com>',
+            'timestamp': 1132617600, # Mon 2005-11-22 00:00:00 +0000
+            'timezone': 0, # UTC
+        }
+        tree.commit('commit 2b', rev_id='2b', **kwargs)
+        tree.commit('commit 3b', rev_id='3b', **kwargs)
+        return tree
+
+    def test_one_revision(self):
+        tree = self.setup_ab_tree()
+        lf = LogCatcher()
+        rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        log.show_log(tree.branch, lf, verbose=True, start_revision=rev,
+                     end_revision=rev)
+        self.assertEqual(1, len(lf.revisions))
+        self.assertEqual(None, lf.revisions[0].revno)   # Out-of-branch
+        self.assertEqual('3a', lf.revisions[0].rev.revision_id)
+
+    def test_many_revisions(self):
+        tree = self.setup_ab_tree()
+        lf = LogCatcher()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        log.show_log(tree.branch, lf, verbose=True, start_revision=start_rev,
+                     end_revision=end_rev)
+        self.assertEqual(3, len(lf.revisions))
+        self.assertEqual(None, lf.revisions[0].revno)   # Out-of-branch
+        self.assertEqual('3a', lf.revisions[0].rev.revision_id)
+        self.assertEqual(None, lf.revisions[1].revno)   # Out-of-branch
+        self.assertEqual('2a', lf.revisions[1].rev.revision_id)
+        self.assertEqual('1', lf.revisions[2].revno)    # In-branch
+
+    def test_long_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+------------------------------------------------------------
+revision-id: 3a
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 3a
+------------------------------------------------------------
+revision-id: 2a
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 2a
+------------------------------------------------------------
+revno: 1
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 1a
+""",
+            tree.branch, log.LongLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
+    def test_short_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+      Joe Foo\t2005-11-22
+      revision-id:3a
+      commit 3a
+
+      Joe Foo\t2005-11-22
+      revision-id:2a
+      commit 2a
+
+    1 Joe Foo\t2005-11-22
+      commit 1a
+
+""",
+            tree.branch, log.ShortLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
+    def test_line_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+Joe Foo 2005-11-22 commit 3a
+Joe Foo 2005-11-22 commit 2a
+1: Joe Foo 2005-11-22 commit 1a
+""",
+            tree.branch, log.LineLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
 
 class TestLogWithBugs(TestCaseForLogFormatter, TestLogMixin):
 
@@ -1688,6 +1874,7 @@ message:
 
 """)
 
+
 class TestLogExcludeAncestry(tests.TestCaseWithTransport):
 
     def make_branch_with_alternate_ancestries(self, relpath='.'):
