@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,11 +19,13 @@ import re
 from bzrlib import (
     bzrdir,
     commands,
+    controldir,
     errors,
     option,
+    registry,
     )
 from bzrlib.builtins import cmd_commit
-from bzrlib.commands import Command, parse_args
+from bzrlib.commands import parse_args
 from bzrlib.tests import TestCase
 from bzrlib.repofmt import knitrepo
 
@@ -107,10 +109,10 @@ class OptionTests(TestCase):
         self.assertFalse(option.Option('foo', hidden=False).is_hidden('foo'))
 
     def test_registry_conversion(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
-        registry.register_metadir('hidden', 'RepositoryFormatKnit1',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two', 'RepositoryFormatKnit1', 'two help')
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormatKnit1',
             'two help', hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', '', registry, str)]
@@ -177,13 +179,13 @@ class OptionTests(TestCase):
         self.assertEqual('test option', my_option.help)
 
     def test_help(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'two help',
             )
-        registry.register_metadir('hidden', 'RepositoryFormat7', 'hidden help',
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormat7', 'hidden help',
             hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
@@ -205,9 +207,9 @@ class OptionTests(TestCase):
         opt = option.Option('hello', help='fg', type=int, argname='gar')
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
                 'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
                 'two help',
                 )
@@ -386,15 +388,23 @@ class TestOptionDefinitions(TestCase):
                     + '\n'.join(msgs))
 
     def test_is_hidden(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('hidden', 'HiddenFormat',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'hidden', 'HiddenFormat',
             'hidden help text', hidden=True)
-        registry.register_metadir('visible', 'VisibleFormat',
+        bzrdir.register_metadir(registry, 'visible', 'VisibleFormat',
             'visible help text', hidden=False)
         format = option.RegistryOption('format', '', registry, str)
         self.assertTrue(format.is_hidden('hidden'))
         self.assertFalse(format.is_hidden('visible'))
 
+    def test_short_name(self):
+        registry = controldir.ControlDirFormatRegistry()
+        opt = option.RegistryOption('format', help='', registry=registry)
+        self.assertEquals(None, opt.short_name())
+        opt = option.RegistryOption('format', short_name='F', help='',
+            registry=registry)
+        self.assertEquals('F', opt.short_name())
+
     def test_option_custom_help(self):
         the_opt = option.Option.OPTIONS['help']
         orig_help = the_opt.help[:]
@@ -403,6 +413,17 @@ class TestOptionDefinitions(TestCase):
         self.assertEqual('suggest lottery numbers', my_opt.help)
         self.assertEqual(orig_help, the_opt.help)
 
+    def test_short_value_switches(self):
+        reg = registry.Registry()
+        reg.register('short', 'ShortChoice')
+        reg.register('long', 'LongChoice')
+        ropt = option.RegistryOption('choice', '', reg, value_switches=True,
+            short_value_switches={'short': 's'})
+        opts, args = parse([ropt], ['--short'])
+        self.assertEqual('ShortChoice', opts.choice)
+        opts, args = parse([ropt], ['-s'])
+        self.assertEqual('ShortChoice', opts.choice)
+
 
 class TestVerboseQuietLinkage(TestCase):
 
