@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for the Registry classes"""
 
@@ -20,14 +20,14 @@ import os
 import sys
 
 from bzrlib import (
-    errors,
-    registry,
+    branch,
     osutils,
+    registry,
+    tests,
     )
-from bzrlib.tests import TestCase, TestCaseInTempDir
 
 
-class TestRegistry(TestCase):
+class TestRegistry(tests.TestCase):
 
     def register_stuff(self, a_registry):
         a_registry.register('one', 1)
@@ -39,9 +39,9 @@ class TestRegistry(TestCase):
         a_registry = registry.Registry()
         self.register_stuff(a_registry)
 
-        self.failUnless(a_registry.default_key is None)
+        self.assertTrue(a_registry.default_key is None)
 
-        # test get() (self.default_key == None)
+        # test get() (self.default_key is None)
         self.assertRaises(KeyError, a_registry.get)
         self.assertRaises(KeyError, a_registry.get, None)
         self.assertEqual(2, a_registry.get('two'))
@@ -49,7 +49,7 @@ class TestRegistry(TestCase):
 
         # test _set_default_key
         a_registry.default_key = 'five'
-        self.failUnless(a_registry.default_key == 'five')
+        self.assertTrue(a_registry.default_key == 'five')
         self.assertEqual(5, a_registry.get())
         self.assertEqual(5, a_registry.get(None))
         # If they ask for a specific entry, they should get KeyError
@@ -64,9 +64,9 @@ class TestRegistry(TestCase):
         a_registry = registry.Registry()
         self.register_stuff(a_registry)
 
-        self.failUnless('one' in a_registry)
+        self.assertTrue('one' in a_registry)
         a_registry.remove('one')
-        self.failIf('one' in a_registry)
+        self.assertFalse('one' in a_registry)
         self.assertRaises(KeyError, a_registry.get, 'one')
 
         a_registry.register('one', 'one')
@@ -202,7 +202,66 @@ class TestRegistry(TestCase):
         self.assertIs(sftp_object, found_object)
 
 
-class TestRegistryWithDirs(TestCaseInTempDir):
+class TestRegistryIter(tests.TestCase):
+    """Test registry iteration behaviors.
+
+    There are dark corner cases here when the registered objects trigger
+    addition in the iterated registry.
+    """
+
+    def setUp(self):
+        super(TestRegistryIter, self).setUp()
+
+        # We create a registry with "official" objects and "hidden"
+        # objects. The later represent the side effects that led to bug #277048
+        # and #430510
+        _registry = registry.Registry()
+
+        def register_more():
+           _registry.register('hidden', None)
+
+        # Avoid closing over self by binding local variable
+        self.registry = _registry
+        self.registry.register('passive', None)
+        self.registry.register('active', register_more)
+        self.registry.register('passive-too', None)
+
+        class InvasiveGetter(registry._ObjectGetter):
+
+            def get_obj(inner_self):
+                # Surprise ! Getting a registered object (think lazy loaded
+                # module) register yet another object !
+                _registry.register('more hidden', None)
+                return inner_self._obj
+
+        self.registry.register('hacky', None)
+        # We peek under the covers because the alternative is to use lazy
+        # registration and create a module that can reference our test registry
+        # it's too much work for such a corner case -- vila 090916
+        self.registry._dict['hacky'] = InvasiveGetter(None)
+
+    def _iter_them(self, iter_func_name):
+        iter_func = getattr(self.registry, iter_func_name, None)
+        self.assertIsNot(None, iter_func)
+        count = 0
+        for name, func in iter_func():
+            count += 1
+            self.assertFalse(name in ('hidden', 'more hidden'))
+            if func is not None:
+                # Using an object register another one as a side effect
+                func()
+        self.assertEqual(4, count)
+
+    def test_iteritems(self):
+        # the dict is modified during the iteration
+        self.assertRaises(RuntimeError, self._iter_them, 'iteritems')
+
+    def test_items(self):
+        # we should be able to iterate even if one item modify the dict
+        self._iter_them('items')
+
+
+class TestRegistryWithDirs(tests.TestCaseInTempDir):
     """Registry tests that require temporary dirs"""
 
     def create_plugin_file(self, contents):
@@ -229,6 +288,13 @@ class TestRegistryWithDirs(TestCaseInTempDir):
             '\n\n'
         )
 
+    def test_lazy_import_registry_foo(self):
+        a_registry = registry.Registry()
+        a_registry.register_lazy('foo', 'bzrlib.branch', 'Branch')
+        a_registry.register_lazy('bar', 'bzrlib.branch', 'Branch.hooks')
+        self.assertEqual(branch.Branch, a_registry.get('foo'))
+        self.assertEqual(branch.Branch.hooks, a_registry.get('bar'))
+
     def test_lazy_import_registry(self):
         plugin_name = self.create_simple_plugin()
         a_registry = registry.Registry()
@@ -240,7 +306,7 @@ class TestRegistryWithDirs(TestCaseInTempDir):
         self.assertEqual(['function', 'klass', 'module', 'obj'],
                          sorted(a_registry.keys()))
         # The plugin should not be loaded until we grab the first object
-        self.failIf(plugin_name in sys.modules)
+        self.assertFalse(plugin_name in sys.modules)
 
         # By default the plugin won't be in the search path
         self.assertRaises(ImportError, a_registry.get, 'obj')
@@ -250,7 +316,7 @@ class TestRegistryWithDirs(TestCaseInTempDir):
         try:
             obj = a_registry.get('obj')
             self.assertEqual('foo', obj)
-            self.failUnless(plugin_name in sys.modules)
+            self.assertTrue(plugin_name in sys.modules)
 
             # Now grab another object
             func = a_registry.get('function')
@@ -274,3 +340,17 @@ class TestRegistryWithDirs(TestCaseInTempDir):
         finally:
             sys.path.remove(plugin_path)
 
+    def test_lazy_import_get_module(self):
+        a_registry = registry.Registry()
+        a_registry.register_lazy('obj', "bzrlib.tests.test_registry",
+            'object1')
+        self.assertEquals("bzrlib.tests.test_registry",
+            a_registry._get_module("obj"))
+
+    def test_normal_get_module(self):
+        class AThing(object):
+            """Something"""
+        a_registry = registry.Registry()
+        a_registry.register("obj", AThing())
+        self.assertEquals("bzrlib.tests.test_registry",
+            a_registry._get_module("obj"))
