@@ -419,6 +419,8 @@ class TestUrlToPath(TestCase):
             from_url('file:///d:/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
         self.assertEqual('/', from_url('file:///'))
 
+        self.assertRaises(InvalidURL, from_url, 'file:///C:')
+        self.assertRaises(InvalidURL, from_url, 'file:///c')
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
         self.assertRaises(InvalidURL, from_url, 'file:///path/to/foo')
