@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,39 +18,16 @@ from cStringIO import StringIO
 
 from bzrlib import (
     errors,
+    fifo_cache,
     inventory,
     xml6,
     xml7,
     xml8,
-    serializer,
     )
 from bzrlib.tests import TestCase
-from bzrlib.inventory import Inventory, InventoryEntry
-from bzrlib.xml4 import serializer_v4
+from bzrlib.inventory import Inventory
 import bzrlib.xml5
 
-_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
-<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
-<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
-<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
-</inventory>"""
-
-
-_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
-    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
-    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    timestamp="1125907235.212"
-    timezone="36000">
-<message>- start splitting code for xml (de)serialization away from objects
-  preparatory to supporting multiple formats by a single library
-</message>
-<parents>
-<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
-</parents>
-</revision>
-"""
-
 _revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
     inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
     revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
@@ -217,26 +194,6 @@ _inventory_v5b = """<inventory format="5" revision_id="a-rev-id">
 class TestSerializer(TestCase):
     """Test XML serialization"""
 
-    def test_canned_inventory(self):
-        """Test unpacked a canned inventory v4 file."""
-        inp = StringIO(_working_inventory_v4)
-        inv = serializer_v4.read_inventory(inp)
-        self.assertEqual(len(inv), 4)
-        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
-
-    def test_unpack_revision(self):
-        """Test unpacking a canned revision v4"""
-        inp = StringIO(_revision_v4)
-        rev = serializer_v4.read_revision(inp)
-        eq = self.assertEqual
-        eq(rev.committer,
-           "Martin Pool <mbp@sourcefrog.net>")
-        eq(rev.inventory_id,
-           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
-        eq(len(rev.parent_ids), 1)
-        eq(rev.parent_ids[0],
-           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
-
     def test_unpack_revision_5(self):
         """Test unpacking a canned revision v5"""
         inp = StringIO(_revision_v5)
@@ -290,6 +247,38 @@ class TestSerializer(TestCase):
                 _inventory_v5a, revision_id='test-rev-id')
         self.assertEqual('test-rev-id', inv.root.revision)
 
+    def test_unpack_inventory_5a_cache_and_copy(self):
+        # Passing an entry_cache should get populated with the objects
+        # But the returned objects should be copies if return_from_cache is
+        # False
+        entry_cache = fifo_cache.FIFOCache()
+        inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(
+            _inventory_v5a, revision_id='test-rev-id',
+            entry_cache=entry_cache, return_from_cache=False)
+        for entry in inv.iter_just_entries():
+            key = (entry.file_id, entry.revision)
+            if entry.file_id is inv.root.file_id:
+                # The root id is inferred for xml v5
+                self.assertFalse(key in entry_cache)
+            else:
+                self.assertIsNot(entry, entry_cache[key])
+
+    def test_unpack_inventory_5a_cache_no_copy(self):
+        # Passing an entry_cache should get populated with the objects
+        # The returned objects should be exact if return_from_cache is
+        # True
+        entry_cache = fifo_cache.FIFOCache()
+        inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(
+            _inventory_v5a, revision_id='test-rev-id',
+            entry_cache=entry_cache, return_from_cache=True)
+        for entry in inv.iter_just_entries():
+            key = (entry.file_id, entry.revision)
+            if entry.file_id is inv.root.file_id:
+                # The root id is inferred for xml v5
+                self.assertFalse(key in entry_cache)
+            else:
+                self.assertIs(entry, entry_cache[key])
+
     def test_unpack_inventory_5b(self):
         inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(
                 _inventory_v5b, revision_id='test-rev-id')
@@ -547,3 +536,10 @@ class TestEncodeAndEscape(TestCase):
         uni_str = u'\xb5\xe5&\u062c'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
                          bzrlib.xml8._encode_and_escape(uni_str))
+
+
+class TestMisc(TestCase):
+
+    def test_unescape_xml(self):
+        """We get some kind of error when malformed entities are passed"""
+        self.assertRaises(KeyError, bzrlib.xml8._unescape_xml, 'foo&bar;')
