@@ -30,6 +30,7 @@ import weakref
 from bzrlib import (
     debug,
     errors,
+    transport,
     ui,
     urlutils,
     )
@@ -41,7 +42,6 @@ from bzrlib.trace import mutter
 from bzrlib.transport import (
     ConnectedTransport,
     _CoalescedOffset,
-    get_transport,
     Transport,
     )
 
@@ -74,7 +74,7 @@ def extract_auth(url, password_manager):
             password = urllib.unquote(password)
         else:
             password = ui.ui_factory.get_password(
-                prompt='HTTP %(user)s@%(host)s password',
+                prompt=u'HTTP %(user)s@%(host)s password',
                 user=username, host=host)
         password_manager.add_password(None, host, username, password)
     url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
@@ -271,7 +271,7 @@ class HttpTransportBase(ConnectedTransport):
                         cur_offset_and_size = iter_offsets.next()
 
             except (errors.ShortReadvError, errors.InvalidRange,
-                    errors.InvalidHttpRange), e:
+                    errors.InvalidHttpRange, errors.HttpBoundaryMissing), e:
                 mutter('Exception %r: %s during http._readv',e, e)
                 if (not isinstance(e, errors.ShortReadvError)
                     or retried_offset == cur_offset_and_size):
@@ -570,14 +570,14 @@ class HttpTransportBase(ConnectedTransport):
                                             self._user, self._password,
                                             host, port,
                                             base_path)
-                new_transport = get_transport(new_url)
+                new_transport = transport.get_transport(new_url)
         else:
             # Redirected to a different protocol
             new_url = self._unsplit_url(scheme,
                                         user, password,
                                         host, port,
                                         base_path)
-            new_transport = get_transport(new_url)
+            new_transport = transport.get_transport(new_url)
         return new_transport
 
 
@@ -629,6 +629,11 @@ class SmartClientHTTPMedium(medium.SmartClientMedium):
         """
         pass
 
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        t = self._http_transport_ref()
+        t.disconnect()
+
 
 # TODO: May be better located in smart/medium.py with the other
 # SmartMediumRequest classes
@@ -661,3 +666,14 @@ class SmartClientHTTPMediumRequest(medium.SmartClientMediumRequest):
     def _finished_reading(self):
         """See SmartClientMediumRequest._finished_reading."""
         pass
+
+
+def unhtml_roughly(maybe_html, length_limit=1000):
+    """Very approximate html->text translation, for presenting error bodies.
+
+    :param length_limit: Truncate the result to this many characters.
+
+    >>> unhtml_roughly("<b>bad</b> things happened\\n")
+    ' bad  things happened '
+    """
+    return re.subn(r"(<[^>]*>|\n|&nbsp;)", " ", maybe_html)[0][:length_limit]
