@@ -114,12 +114,6 @@ class SFTPLock(object):
         except FileExists:
             raise LockError('File %r already locked' % (self.path,))
 
-    def __del__(self):
-        """Should this warn, or actually try to cleanup?"""
-        if self.lock_file:
-            warning("SFTPLock %r not explicitly unlocked" % (self.path,))
-            self.unlock()
-
     def unlock(self):
         if not self.lock_file:
             return
@@ -281,6 +275,8 @@ class _SFTPReadvHelper(object):
                     buffered = buffered[buffered_offset:]
                     buffered_data = [buffered]
                     buffered_len = len(buffered)
+        # now that the data stream is done, close the handle
+        fp.close()
         if buffered_len:
             buffered = ''.join(buffered_data)
             del buffered_data[:]
@@ -389,6 +385,11 @@ class SFTPTransport(ConnectedTransport):
                                          self._host, self._port)
         return connection, (user, password)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get_sftp(self):
         """Ensures that a connection is established"""
         connection = self._get_connection()
@@ -416,11 +417,6 @@ class SFTPTransport(ConnectedTransport):
         :param relpath: The relative path to the file
         """
         try:
-            # FIXME: by returning the file directly, we don't pass this
-            # through to report_activity.  We could try wrapping the object
-            # before it's returned.  For readv and get_bytes it's handled in
-            # the higher-level function.
-            # -- mbp 20090126
             path = self._remote_path(relpath)
             f = self._get_sftp().file(path, mode='rb')
             if self._do_prefetch and (getattr(f, 'prefetch', None) is not None):
@@ -715,6 +711,8 @@ class SFTPTransport(ConnectedTransport):
             if (e.args[0].startswith('Directory not empty: ')
                 or getattr(e, 'errno', None) == errno.ENOTEMPTY):
                 raise errors.DirectoryNotEmpty(path, str(e))
+            if e.args == ('Operation unsupported',):
+                raise errors.TransportNotPossible()
             mutter('Raising exception with args %s', e.args)
         if getattr(e, 'errno', None) is not None:
             mutter('Raising exception with errno %s', e.errno)
