@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,13 +31,14 @@ lazy_import(globals(), """
 import errno
 import stat
 
-import bzrlib
 from bzrlib import (
     bzrdir,
     cache_utf8,
+    conflicts as _mod_conflicts,
     debug,
     dirstate,
     errors,
+    filters as _mod_filters,
     generate_ids,
     osutils,
     revision as _mod_revision,
@@ -46,14 +47,13 @@ from bzrlib import (
     transform,
     views,
     )
-import bzrlib.branch
-import bzrlib.ui
 """)
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.filters import filtered_input_file, internal_size_sha_file_byname
 from bzrlib.inventory import Inventory, ROOT_ID, entry_factory
 from bzrlib.lock import LogicalLockResult
+from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import (
     file_kind,
@@ -62,14 +62,22 @@ from bzrlib.osutils import (
     realpath,
     safe_unicode,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport.local import LocalTransport
-from bzrlib.tree import InterTree
-from bzrlib.tree import Tree
-from bzrlib.workingtree import WorkingTree, WorkingTree3, WorkingTreeFormat3
+from bzrlib.tree import (
+    InterTree,
+    InventoryTree,
+    )
+from bzrlib.workingtree import (
+    InventoryWorkingTree,
+    WorkingTree,
+    WorkingTreeFormat,
+    )
 
 
-class DirStateWorkingTree(WorkingTree3):
+class DirStateWorkingTree(InventoryWorkingTree):
+
+    _DEFAULT_WORTH_SAVING_LIMIT = 10
+
     def __init__(self, basedir,
                  branch,
                  _control_files=None,
@@ -85,7 +93,7 @@ class DirStateWorkingTree(WorkingTree3):
         self._format = _format
         self.bzrdir = _bzrdir
         basedir = safe_unicode(basedir)
-        mutter("opening working tree %r", basedir)
+        trace.mutter("opening working tree %r", basedir)
         self._branch = branch
         self.basedir = realpath(basedir)
         # if branch is at our basedir and is a format 6 or less
@@ -125,6 +133,10 @@ class DirStateWorkingTree(WorkingTree3):
             state.add(f, file_id, kind, None, '')
         self._make_dirty(reset_inventory=True)
 
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
     def _make_dirty(self, reset_inventory):
         """Make the tree state dirty.
 
@@ -182,7 +194,7 @@ class DirStateWorkingTree(WorkingTree3):
 
     def _comparison_data(self, entry, path):
         kind, executable, stat_value = \
-            WorkingTree3._comparison_data(self, entry, path)
+            WorkingTree._comparison_data(self, entry, path)
         # it looks like a plain directory, but it's really a reference -- see
         # also kind()
         if (self._repo_supports_tree_reference and kind == 'directory'
@@ -194,7 +206,7 @@ class DirStateWorkingTree(WorkingTree3):
     def commit(self, message=None, revprops=None, *args, **kwargs):
         # mark the tree as dirty post commit - commit
         # can change the current versioned list by doing deletes.
-        result = WorkingTree3.commit(self, message, revprops, *args, **kwargs)
+        result = WorkingTree.commit(self, message, revprops, *args, **kwargs)
         self._make_dirty(reset_inventory=True)
         return result
 
@@ -219,7 +231,7 @@ class DirStateWorkingTree(WorkingTree3):
         local_path = self.bzrdir.get_workingtree_transport(None
             ).local_abspath('dirstate')
         self._dirstate = dirstate.DirState.on_file(local_path,
-            self._sha1_provider())
+            self._sha1_provider(), self._worth_saving_limit())
         return self._dirstate
 
     def _sha1_provider(self):
@@ -234,6 +246,26 @@ class DirStateWorkingTree(WorkingTree3):
         else:
             return None
 
+    def _worth_saving_limit(self):
+        """How many hash changes are ok before we must save the dirstate.
+
+        :return: an integer. -1 means never save.
+        """
+        config = self.branch.get_config()
+        val = config.get_user_option('bzr.workingtree.worth_saving_limit')
+        if val is None:
+            val = self._DEFAULT_WORTH_SAVING_LIMIT
+        else:
+            try:
+                val = int(val)
+            except ValueError, e:
+                trace.warning('Invalid config value for'
+                              ' "bzr.workingtree.worth_saving_limit"'
+                              ' value %r is not an integer.'
+                              % (val,))
+                val = self._DEFAULT_WORTH_SAVING_LIMIT
+        return val
+
     def filter_unversioned_files(self, paths):
         """Filter out paths that are versioned.
 
@@ -369,7 +401,7 @@ class DirStateWorkingTree(WorkingTree3):
         state = self.current_dirstate()
         if stat_value is None:
             try:
-                stat_value = os.lstat(file_abspath)
+                stat_value = osutils.lstat(file_abspath)
             except OSError, e:
                 if e.errno == errno.ENOENT:
                     return None
@@ -478,7 +510,7 @@ class DirStateWorkingTree(WorkingTree3):
             self._must_be_locked()
             if not path:
                 path = self.id2path(file_id)
-            mode = os.lstat(self.abspath(path)).st_mode
+            mode = osutils.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
     def all_file_ids(self):
@@ -850,7 +882,7 @@ class DirStateWorkingTree(WorkingTree3):
                 rollback_rename()
                 raise
             result.append((from_rel, to_rel))
-            state._dirblock_state = dirstate.DirState.IN_MEMORY_MODIFIED
+            state._mark_modified()
             self._make_dirty(reset_inventory=False)
 
         return result
@@ -1113,7 +1145,29 @@ class DirStateWorkingTree(WorkingTree3):
                         _mod_revision.NULL_REVISION)))
                 ghosts.append(rev_id)
             accepted_revisions.add(rev_id)
-        dirstate.set_parent_trees(real_trees, ghosts=ghosts)
+        updated = False
+        if (len(real_trees) == 1
+            and not ghosts
+            and self.branch.repository._format.fast_deltas
+            and isinstance(real_trees[0][1],
+                revisiontree.InventoryRevisionTree)
+            and self.get_parent_ids()):
+            rev_id, rev_tree = real_trees[0]
+            basis_id = self.get_parent_ids()[0]
+            # There are times when basis_tree won't be in
+            # self.branch.repository, (switch, for example)
+            try:
+                basis_tree = self.branch.repository.revision_tree(basis_id)
+            except errors.NoSuchRevision:
+                # Fall back to the set_parent_trees(), since we can't use
+                # _make_delta if we can't get the RevisionTree
+                pass
+            else:
+                delta = rev_tree.inventory._make_delta(basis_tree.inventory)
+                dirstate.update_basis_by_delta(delta, rev_id)
+                updated = True
+        if not updated:
+            dirstate.set_parent_trees(real_trees, ghosts=ghosts)
         self._make_dirty(reset_inventory=False)
 
     def _set_root_id(self, file_id):
@@ -1139,10 +1193,10 @@ class DirStateWorkingTree(WorkingTree3):
 
     def unlock(self):
         """Unlock in format 4 trees needs to write the entire dirstate."""
-        # do non-implementation specific cleanup
-        self._cleanup()
-
         if self._control_files._lock_count == 1:
+            # do non-implementation specific cleanup
+            self._cleanup()
+
             # eventually we should do signature checking during read locks for
             # dirstate updates.
             if self._control_files._lock_mode == 'w':
@@ -1254,7 +1308,7 @@ class DirStateWorkingTree(WorkingTree3):
     def rename_one(self, from_rel, to_rel, after=False):
         """See WorkingTree.rename_one"""
         self.flush()
-        WorkingTree.rename_one(self, from_rel, to_rel, after)
+        super(DirStateWorkingTree, self).rename_one(from_rel, to_rel, after)
 
     @needs_tree_write_lock
     def apply_inventory_delta(self, changes):
@@ -1293,6 +1347,27 @@ class DirStateWorkingTree(WorkingTree3):
             self._inventory = inv
         self.flush()
 
+    @needs_tree_write_lock
+    def reset_state(self, revision_ids=None):
+        """Reset the state of the working tree.
+
+        This does a hard-reset to a last-known-good state. This is a way to
+        fix if something got corrupted (like the .bzr/checkout/dirstate file)
+        """
+        if revision_ids is None:
+            revision_ids = self.get_parent_ids()
+        if not revision_ids:
+            base_tree = self.branch.repository.revision_tree(
+                _mod_revision.NULL_REVISION)
+            trees = []
+        else:
+            trees = zip(revision_ids,
+                        self.branch.repository.revision_trees(revision_ids))
+            base_tree = trees[0][1]
+        state = self.current_dirstate()
+        # We don't support ghosts yet
+        state.set_state_from_scratch(base_tree.inventory, trees, [])
+
 
 class ContentFilterAwareSHA1Provider(dirstate.SHA1Provider):
 
@@ -1303,7 +1378,7 @@ class ContentFilterAwareSHA1Provider(dirstate.SHA1Provider):
         """See dirstate.SHA1Provider.sha1()."""
         filters = self.tree._content_filter_stack(
             self.tree.relpath(osutils.safe_unicode(abspath)))
-        return internal_size_sha_file_byname(abspath, filters)[1]
+        return _mod_filters.internal_size_sha_file_byname(abspath, filters)[1]
 
     def stat_and_sha1(self, abspath):
         """See dirstate.SHA1Provider.stat_and_sha1()."""
@@ -1313,7 +1388,7 @@ class ContentFilterAwareSHA1Provider(dirstate.SHA1Provider):
         try:
             statvalue = os.fstat(file_obj.fileno())
             if filters:
-                file_obj = filtered_input_file(file_obj, filters)
+                file_obj = _mod_filters.filtered_input_file(file_obj, filters)
             sha1 = osutils.size_sha_file(file_obj)[1]
         finally:
             file_obj.close()
@@ -1349,7 +1424,7 @@ class ContentFilteringDirStateWorkingTree(DirStateWorkingTree):
 class WorkingTree4(DirStateWorkingTree):
     """This is the Format 4 working tree.
 
-    This differs from WorkingTree3 by:
+    This differs from WorkingTree by:
      - Having a consolidated internal dirstate, stored in a
        randomly-accessible sorted file on disk.
      - Not having a regular inventory attribute.  One can be synthesized
@@ -1383,14 +1458,26 @@ class WorkingTree6(ContentFilteringDirStateWorkingTree):
         return views.PathBasedViews(self)
 
 
-class DirStateWorkingTreeFormat(WorkingTreeFormat3):
+class DirStateWorkingTreeFormat(WorkingTreeFormat):
+
+    missing_parent_conflicts = True
+
+    supports_versioned_directories = True
+
+    _lock_class = LockDir
+    _lock_file_name = 'lock'
+
+    def _open_control_files(self, a_bzrdir):
+        transport = a_bzrdir.get_workingtree_transport(None)
+        return LockableFiles(transport, self._lock_file_name,
+                             self._lock_class)
 
     def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """See WorkingTreeFormat.initialize().
 
         :param revision_id: allows creating a working tree at a different
-        revision than the branch is at.
+            revision than the branch is at.
         :param accelerator_tree: A tree which can be used for retrieving file
             contents more quickly than the revision tree, i.e. a workingtree.
             The revision tree will be used for cases where accelerator_tree's
@@ -1489,6 +1576,20 @@ class DirStateWorkingTreeFormat(WorkingTreeFormat3):
         :param wt: the WorkingTree object
         """
 
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        wt = self._open(a_bzrdir, self._open_control_files(a_bzrdir))
+        return wt
+
     def _open(self, a_bzrdir, control_files):
         """Open the tree itself.
 
@@ -1585,7 +1686,7 @@ class WorkingTreeFormat6(DirStateWorkingTreeFormat):
         return True
 
 
-class DirStateRevisionTree(Tree):
+class DirStateRevisionTree(InventoryTree):
     """A revision tree pulling the inventory from a dirstate.
     
     Note that this is one of the historical (ie revision) trees cached in the
@@ -1610,13 +1711,10 @@ class DirStateRevisionTree(Tree):
     def annotate_iter(self, file_id,
                       default_revision=_mod_revision.CURRENT_REVISION):
         """See Tree.annotate_iter"""
-        text_key = (file_id, self.inventory[file_id].revision)
+        text_key = (file_id, self.get_file_revision(file_id))
         annotations = self._repository.texts.annotate(text_key)
         return [(key[-1], line) for (key, line) in annotations]
 
-    def _get_ancestors(self, default_revision):
-        return set(self._repository.get_ancestry(self._revision_id,
-                                                 topo_sorted=False))
     def _comparison_data(self, entry, path):
         """See Tree._comparison_data."""
         if entry is None:
@@ -1738,8 +1836,6 @@ class DirStateRevisionTree(Tree):
                 elif kind == 'directory':
                     parent_ies[(dirname + '/' + name).strip('/')] = inv_entry
                 elif kind == 'symlink':
-                    inv_entry.executable = False
-                    inv_entry.text_size = None
                     inv_entry.symlink_target = utf8_decode(fingerprint)[0]
                 elif kind == 'tree-reference':
                     inv_entry.reference_revision = fingerprint or None
@@ -1782,6 +1878,10 @@ class DirStateRevisionTree(Tree):
             return parent_details[1]
         return None
 
+    @needs_read_lock
+    def get_file_revision(self, file_id):
+        return self.inventory[file_id].revision
+
     def get_file(self, file_id, path=None):
         return StringIO(self.get_file_text(file_id))
 
@@ -1810,7 +1910,7 @@ class DirStateRevisionTree(Tree):
                                        identifier))
         return self._repository.iter_files_bytes(repo_desired_files)
 
-    def get_symlink_target(self, file_id):
+    def get_symlink_target(self, file_id, path=None):
         entry = self._get_entry(file_id=file_id)
         parent_index = self._get_parent_index()
         if entry[1][parent_index][0] != 'l':
@@ -1869,7 +1969,7 @@ class DirStateRevisionTree(Tree):
     def is_executable(self, file_id, path=None):
         ie = self.inventory[file_id]
         if ie.kind != "file":
-            return None
+            return False
         return ie.executable
 
     def is_locked(self):
@@ -1988,7 +2088,7 @@ class InterDirStateTree(InterTree):
     def make_source_parent_tree(source, target):
         """Change the source tree into a parent of the target."""
         revid = source.commit('record tree')
-        target.branch.repository.fetch(source.branch.repository, revid)
+        target.branch.fetch(source.branch, revid)
         target.set_parent_ids([revid])
         return target.basis_tree(), target
 
